/***************************************************************************
 ** $Id: KLNcanvasitems.cpp,v 1.12 2004/05/24 21:29:37 silhusk Exp $
                          klearnnotes2
                          KLNcanvasitems.cpp -  description
                             -------------------
    begin                : Wed Oct  8 15:32:54 CEST 2003
    copyright            : (C) 2003 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "KLNcanvasitems.h"
#include "basemusicnote.h"
#include "scalenotation.h"
#include <qtimer.h>
#include <qevent.h>
#include <qwmatrix.h> 
#include <qpainter.h> 
#include <qtranslator.h>

//!middle point of the canvas
#define  MIDDLEPOINT 17*SC


//////////// ********** aNOTE  *************

Note::Note (int zeroH, QCanvas* mycanvas, int mynumber, 
	    StaffLine* la[], StaffLine* lb[],
	    QObject* parent,Note* prevN) 
  :  QObject(parent) , QCanvasEllipse ( 4*SC, 2*SC, mycanvas ),
     RandomObject()
{
  canv=mycanvas;
  num=mynumber;
  previousNote=prevN;

  shifted=false;
  //QCanvasEllipse setup:
  hide();
  color='0';
  resetcolor='0';

  midx=MIDDLEPOINT;
  QCanvasEllipse::move( MIDDLEPOINT, ((num-1)*SC)+zeroH);
  // now this is fun: there has to be order of Z-s
  // each note is antialiased and fills all region between lines
  // it has to be over a line which crosses it in the middle,
  // but below line touched above (otherwise antialias prepared
  // for white background will look stupid on a black line);
  // therefore there has to be Z-order:
  // a line with smallest Z
  // then a note over this line (*)
  // then a note just above the line
  // then a line one step above
  // then the note over this line (*) and so on


  // OOPS, eventually I made smaller note-pictures so above remarks
  // DO NOT APPLY!; but I don't delete them, because it may be important
  // in the future


  int k = int(y()/double(SC)); //how many steps are we from tops of the canvas
  setZ(128-(2*k)+1);// +1 will put it over "it's" line but below
  //                   items higher at the stave

  noOfMyLines=0;
  //extra lines:
  if (num<0){ // we are above note G2
    noOfMyLines=(1-num)/2; //which hopefully is floored for an odd numerator
    for (int i =0 ; i<noOfMyLines; i++)
      myLines[i]=la[i];
  }
  if (num>10){ //we are below D
    noOfMyLines=(num-9)/2;
    for(int i = 0; i<noOfMyLines; i++)
      myLines[i]=lb[i];
  }

  cnameLetter=new QCanvasText(" ", mycanvas );
  cnameLetter->setFont( QFont( "Arial", int(1.5*SC), QFont::Bold ) );
  cnameLetter->setColor( QColor( 0, 0, 0 ) );
  cnameLetter->hide();
  cnameLetter->setZ(200);

  cnameLetterBG=new QCanvasRectangle(mycanvas);
  cnameLetterBG->setSize(cnameLetter->boundingRect().width()+2,
			 cnameLetter->boundingRect().height()-2);
  cnameLetterBG->setBrush(QBrush(QColor(255,255,255)));
  cnameLetterBG->setPen(QPen (QColor (255,255,255),0));
  cnameLetterBG->setZ(cnameLetter->z()-1);
  cnameLetterBG->hide();
  //  textshift=0;
  //  for(int i = 0; i<noOfMyLines; i++)
  //    myLines[i]->moveX(x());
  move(midx,y());// to update cnameLetter position
}

void Note::move(double newx,double newy)
{
  // redefined just to move the cnameLetter too!
  //  DEBUG_VAR(newx);
  midx=int(newx);
  if(shifted)
    if(num%2)
      {
	//	textshift=1;
	QCanvasEllipse::move( newx-1.75*float(SC), newy);
	cnameLetter->move( newx-4*SC-cnameLetter->boundingRect().width(),
			   newy-SC+1);
      }
    else
      {
	QCanvasEllipse::move( newx+1.75*float(SC), newy);
	cnameLetter->move( newx+4*SC,newy-SC+1);
      }
  else // not shifted:
    {
      QCanvasEllipse::move( newx, newy);
      cnameLetter->move( newx+2*SC,newy-SC+1);
    }
  cnameLetterBG->move(cnameLetter->x()-1,cnameLetter->y()+1);
  for(int i = 0; i<noOfMyLines; i++)
    myLines[i]->moveX(newx);
  canv->update();
}

void Note::advance()
{
  double oldx=x();
  QCanvasEllipse::advance(1);
  move(midx+x()-oldx,y());
}

Note::~Note() {}

void Note::setCnameLetterColor(char newcolor)
{
  switch (newcolor){
  case 'R':
    cnameLetter->setColor( QColor( 127, 0, 0 ) );
    break;
  case 'G':
    cnameLetter->setColor( QColor( 0, 127, 0 ) );
    break;
  case 'B':
    cnameLetter->setColor( QColor( 16, 34, 188 ) );
    break;
  case 'S':
    cnameLetter->setColor( QColor( 187,102, 0 ) );
    break;
  case '0':
    cnameLetter->setColor( QColor( 0, 0, 0 ) );
    break;
  }
}

void Note::setColorTmp(char newcolor,int time)
{
  color=newcolor;
  setCnameLetterColor(newcolor);
  update();
  cnameLetter->show();
  cnameLetterBG->show();
  QTimer::singleShot( time, this, SLOT(resetColor()) );
  return;
}

void Note::setColor(char newcolor)
{
  color=newcolor;
  setCnameLetterColor(newcolor);
  resetcolor=newcolor;
  update();
  return;
}


void Note::resetColor()
{
  color=resetcolor;
  setCnameLetterColor(resetcolor);
  update();
  cnameLetter->hide();
  cnameLetterBG->hide();
  canv->update();
}

void Note::showNote(int where)
{
  if(where==1) // show in the middle:
    {
      //debuginfo("in the middle");
      move( MIDDLEPOINT, y());
    }
  else // show to the right:
    {
      //debuginfo("to the right");
      move(32*SC, y());
    }

  if (previousNote->isVisible())
    {
      moveit();
      previousNote->moveit();
    }

  show();
  for (int i=0;i<noOfMyLines;i++)
    myLines[i]->show();
}


void Note::hideNote()
{
  hide();
  shifted=false;
  for (int i=0;i<noOfMyLines;i++)
    {
      myLines[i]->hide();
    }
  move( MIDDLEPOINT, y());//move it back
}

void Note::setName(int newname)
{
  name=newname;
  cname=BaseMusicNote::getNotation().getNoteChar(static_cast<ScaleNotation::ScaleNote>(name));
  cnameLetter->setText(BaseMusicNote::getNotation().getNoteLabel(static_cast<ScaleNotation::ScaleNote>(name)));
  cnameLetterBG->setSize(cnameLetter->boundingRect().width()+2,
			 cnameLetter->boundingRect().height()-3);
}

int  Note::getName()	{return name;}
char Note::getCname()	{return cname;}
int  Note::getNum()	{return num;}

QString Note::getStringName()
{
  return BaseMusicNote::getNotation().getNoteLabel(static_cast<ScaleNotation::ScaleNote>(name));
}


void Note::moveit()
{
  for(int i = 0; i<noOfMyLines; i++)
    myLines[i]->setLonger(); // all extra lines longer
  shifted=true;
  move(midx,y());// update()
}


void Note::setPitch(int newpitch)
{
  pitch=newpitch;
}

int Note::getPitch() {return pitch;}

//void Note::play(){ midi_addANote2chord(pitch);}//addAnote checks if device is open

void Note::drawShape ( QPainter & p )
{
  switch (color){
  case 'R':
    p.drawPixmap( int(x() - width()/2), int(y()-height()/2),*PIX_note_red);
    break;
  case 'G':
    p.drawPixmap( int(x() - width()/2), int(y()-height()/2),*PIX_note_green);
    break;
  case 'B':
    p.drawPixmap( int(x() - width()/2), int(y()-height()/2),*PIX_note_blue);
    break;
  case 'S':
    p.drawPixmap( int(x() - width()/2), int(y()-height()/2),*PIX_note_brown);
    break;
  case '0':
    p.drawPixmap( int(x() - width()/2), int(y()-height()/2),*PIX_note_black);
    break;
  }
}


//////////////////////////////////

//bool StaffLine::longer = false;


StaffLine::StaffLine (QCanvas* can,int ypos,int linetype)
  :QCanvasLine(can)// , QObject(can)
{
  longer = false;
  y=ypos;
  type=linetype;
  setPen( QPen( black, 1 ) );
  int k =int(y/double(SC)); //how many steps are we from tops of the canvas
  setZ(128-(2*k));// without +1 this will put it under "it's" note

  switch (type)
    {
    case 1: // one of 5 staff lines
      midX=(35.0/2)*SC; // not used anyway
      setPoints( 2*SC, y, 33*SC, y);
      break;
    case 2: // one of the extra lines above or below
      midX=MIDDLEPOINT;
      setPoints( MIDDLEPOINT-5*SC, y, MIDDLEPOINT+5*SC, y);
      break;
    case 3: // guidelines
      midX=31.5*SC; //not used
      setPoints( 30*SC, y, 33*SC, y);
    }

}
StaffLine::~StaffLine(){}

void StaffLine::show()
{
  if (type==2) // extra lines above/below may need resizing:
    if(longer)
      setPoints( int(midX-5*SC), y, int(midX+5*SC), y);
    else
      setPoints( int(midX-3*SC), y, int(midX+3*SC), y);
  QCanvasLine::show();
}

void StaffLine::hide()
{
  QCanvasLine::hide();
  longer=false;
}

void StaffLine::setLonger(){longer=true;} // ==there are some adjacent notes

void StaffLine::moveX(double newX)
{
  midX=newX;
  if(isVisible())
    show();//updates points
  else
    {
      show();//updates points
      hide();
    }
}

///////////////////////////////////

ClickableView::ClickableView(QCanvas* cnvs,QWidget* prnt, KLearnNotes2* prog)
  :QCanvasView(cnvs,prnt)
{
  papcio=prog;
}

void ClickableView::contentsMousePressEvent ( QMouseEvent * e)
{
  //  debuginfo("ClickableView::contentsMousePressEvent()");
  QPoint p = inverseWorldMatrix().map(e->pos());
  QCanvasItemList l=canvas()->collisions(p);
  for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it) {
    //    debuginfo(QString("Rtti=%1").arg( (*it)->rtti()));
    if ( (*it)->rtti() == QCanvasItem::Rtti_Ellipse ){
      Note *noti= (Note*)(*it);
      noti->setColorTmp('B');
      QCanvasView::canvas()->update();
      papcio->getMidiOutput()->addNoteToChord(noti->getPitch());
      papcio->getMidiOutput()->playChord();
      //      midi_playChord();
      if(!(papcio->isStarted()))
	{
	  papcio->pushButtonChangeColorTmp(noti->getName());//changes button color
	}
      return;
    }
    // ******* DEVEL FEATURES - START
#ifdef CVSFEATURES
    if((*it)->rtti() == PianoKeyboardKey::rttiNum)
      {
	//      debuginfo("PianoKeyboardKey::rttiNum");
	((PianoKeyboardKey*)(*it)) -> clicked();
      }
#endif
    // ******* DEVEL FEATURES - END
  }
}
