/***************************************************************************
 ** $Id: KLNcanvasitems.h,v 1.8 2004/05/13 02:57:19 wiecko Exp $
                          klearnnotes2
                          KLNcanvasitem.h -  description
                             -------------------
    begin                : Wed Oct  8 15:32:54 CEST 2003
    copyright            : (C) 2003 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// ClickableView, StaffLine and Note

#ifndef KLNcanvasitems_H
#define KLNcanvasitems_H


#include <qcanvas.h>
#include "klearnnotes2.h"
#include "randomobject.h"
#include "musicwidget.h"

class KLearnNotes2;
class QPainter;

//! QCanvasView reacting on mouse clicks.
/*! Only redefined mousePressEvent().

But it is aware of KLearnNotes2: it calls KLearnNotes2::checkanswer()
and if a test is not started 
highlights note's namebutton. This will have to
be redefined. ClickableView should just emit a signal which
should be handled within Kln2 class.
*/
class ClickableView: public QCanvasView
{
  Q_OBJECT
    public:
  ClickableView(QCanvas * cnvs, QWidget* parent, KLearnNotes2 * mainwidget);
  public slots:
    //! Checks which note the user clicked on and acts accordingly.
    /*! Right now it calls papcio->checkanswer() and if papcio->isStarted()
      changes color of the clicked note. In future it should only emit a 
      signal which would be handled by papcio's slots instead.
    */
    void contentsMousePressEvent ( QMouseEvent * );
 private:
  KLearnNotes2* papcio;
};







//! A class for all kinds of staff lines.
class StaffLine : public QCanvasLine
{
 public:
  StaffLine (QCanvas*,int ypos,int linetype);
  ~StaffLine();

  //! if(longer) makes lager lines longer; then calls QCanvasLine::show().
  void show();
  //! Calls QCanvasLine::hide() and sets longer=false.
  void hide();

  //! Sets longer=true.
  void setLonger();

  //! moves middle point of the line horizotally to newX
  void moveX(double newX);


 private:
  //! Type of the line (staff-, lager-, guideline).
  /*!
   * -    1= one of the 5 staff lines
   * -    2= an extra staff line above or below the five (a lager line)
   * -    3= a guideline
   */
  int type;   

  //! Vertical position of the line.
  /*! ==endPoint().y() and ==startPoint().y() */
  int y;

  //! horizontal position of the middle point of the line
  double midX;

  //! Should lager lines be made longer?
  /*! Important only for lines of type 2. If two adjacent notes are 
    displayed they are shifted left-right; in this case extra lines 
    should be longer.

    //	This is static: one property variable is used for ALL lager lines
    //	(that is, all lines are longer or all are shorter).
    */
  bool longer;
};






extern QPixmap* PIX_note_red;
extern QPixmap* PIX_note_green;
extern QPixmap* PIX_note_blue;
extern QPixmap* PIX_note_brown;
extern QPixmap* PIX_note_black;

//! a note
/*! Its picture (ellipse), properties (name...) and statistics. */
class Note : public QObject, public QCanvasEllipse, public  RandomObject
{
  Q_OBJECT
    public:
  Note(int zeroheight, QCanvas* mycanvas, int mynumber, 
       StaffLine* la[], StaffLine* lb[],QObject * parent,Note* prevN);
  ~Note();

  //! Shifts note's position left or right slightly.
  /*! Used when two adjacent notes should be displayed - one of them is 
    shifted right, the other left. */
  void moveit();
	
  //! Shows this note and its lines.
  /*! position=1 <=> in the middle; position = 0 <=> to the right */
  void showNote(int position=1); 
  // moves note's picture by one point to the left
  virtual void advance();
  //! Hides this note and its lines.
  void hideNote(); 
  //! Changes note's color to `newcolor' for 'time' microseconds.
  /*! See color and resetcolor properties for some comments. */
  void setColorTmp(char newcolor, int time=INTERVAL); 
  //! Changes note's color to `newcolor' permanently
  /*! See color property for some comments. */
  void setColor(char newcolor); 


  //! Sets name of the note.
  /*! Based on orderOfNotes[]: 0=H/B, 1=A, 2=G ... 6=C.

  Notes are identified by their POSITION in clef which does not change
  (that is: this note == a note on the fifth of the 5 main staff lines;
  another one is there between the 4th and 5th lines and so on).
  This means, for different clefs the same note can have different name.
  Therefore, setName() is run for each note whenever the clef changes.
  */
  void setName(int newname);
  //! Returns integer representing the name of the note (0,...,6)
  int getName();
  //! Returns char name of the note (`B/H','A',...,'C')
  char getCname();
  //! Returns full name of the note ("Ab","C","Do"...)
  QString getStringName();
 
  //! Returns vertical position of the note in the clef.
  /*! The position is set only once when Note is created;
    in (-MAXabove,......,MAXall-MAXabove-1).
  */
  int getNum(); 
  
  //! Sets a MIDI pitch (a tune) of the note.
  /*! Re-run for each note whenever the clef changes */
  void setPitch(int newpitch);
  //! Returns a MIDI pitch (a tune) of the note.
  int getPitch();
  /*//! Adds this note's tune to current chord.*/
  /* The tune is not played yet! Part of the code which runs note->play()
     for each note to be played simultaneously has to  call
     midi_playChord() after all notes were added to the chord.
  */
  /*  void play();*/

  //  public slots:
  //    virtual void update();

  protected slots:
    //! Resets color to previous value after it has been changed temporarily.
    /*! See color property for some comments. */
    void resetColor();
  //! Draws note's picture.
  /*! Overridden QCanvasEllipse::drawShape().

  See color property for some comments. */
  void drawShape ( QPainter & p );

 protected:
  //! is an other nearby note visible? if yes this note should be moved, and lines should be longer
  bool shifted;
  //! unshifted position of the ellipse
  /*! if shifted==true actuall qcanvas elliplse is shown a bit to the
    left or to the right */
  int midx;

  //! Moves both the picture and the canvas letter to a shifted or unshifted position
  /*! arguments are UNSHIFTED positions of the note (i.e. newx will be a new
    value of midx )*/
  void move(double newx, double newy);
  //! Sets color of the note's canvas letter.
  void setCnameLetterColor(char newcolor);
  // Should the canvas letter be to the right or left of the note's picture?
  /* 
   * - 0 if the letter should be displayed to the right of note's picture 
   (the ellipse); 
   * - 1 if the letter is on the left. */
  //  int textshift; 

  //! Integer representation of note's name.
  /*! in (0,...,6)

  See setName() method for some extra comments. */
  int name;
  //! Char representation of note's name.
  /*! in(`C'...'H') */
  char cname; 

  //! Just one letter - the note's name.
  /*! It is displayed in canvas whenever the note is highlighted. */
  QCanvasText* cnameLetter;
  //! Background for cnameLetter.
  /*! Makes cnameLetter 
    displayed clearer - lines around the letter are
    "erased" [=covered by cnameLetterBG
    ].
  */
  QCanvasRectangle* cnameLetterBG;
  //! Vertical position of the note in the staff.
  /*! in (-MAXabove,......,MAXall-MAXabove-1)

  The note just above the toppest of the five main lines has num=0. Then, lower
  notes have higher-and-higher num
  , notes above the five have negative
  num
  . E.g. in treble clef, G1 has num=0, F1 has num=1, E1 has num=2 and so
  on, A1 has num=-1, H/B1 has num=-2 and so on.

  This is quite ugly [higher-tune notes have lower num
  ], but changing this
  would require changes in MANY places of the code.
  */
  int num;    

  //! number of items in myLines[] which are used for this note
  int noOfMyLines;
  //! Lager (extra) lines which "belong" to this note.
  /*! If a note is above or below the main five staff lines, whenever it is
    shown a number of extra (lager) lines have to be shown too. 
    myLines[] array hold all such lines for a note.

    For example, for C2 note in treble clef myLines[] would hold two 
    lines: "A1" and "C2" lines - the two just above the staff.

    The size of an array is a little waste, but it's hard to
    determine which of MAXabove and MAXbelow is greater  So, it is
    a waste but it is safe.
  */
  StaffLine* myLines[defMAXall];  
  //! the parent canvas
  QCanvas* canv;
  //! previous note (the one just above this one)
  /*! Used for displaying: if previous note is visible and this one is shown,
    both this one and the previous one should be shifted left or right.

    If previousNote->isVisible(), this->moveit() and previousNote->moveit() 
    are called.
  */
  Note* previousNote;
  //! Note's MIDI pitch (tune).
  int pitch;

  //! Color of the note.
  /*!
   * - `R' = red
   * - `G' = green
   * - `B' = blue (!)
   * - `S' = brown
   * - `0' (zero) = black(!).

   Used by drawShape() to decide which pixmap to use.
  */
  char color;
  //! A color used by resetColor().
  /*! When setColorTmp() is run, it stores the current color in resetcolor
    variable and changes color to a new one. Then, after some time
    the color is shifted back to resetcolor.
  */
  char resetcolor;
};
#endif
