/***************************************************************************
 ** $Id: KLNfancywidgets.cpp,v 1.6 2004/04/27 02:28:22 wiecko Exp $
                          klearnnotes2
                          KLNfancywidgets.cpp -  description
                             -------------------
    begin                : Wed Oct  8 15:32:54 CEST 2003
    copyright            : (C) 2003 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qobject.h>
#include <qpixmap.h>
#include <qcolor.h>
#include <qtimer.h>
#include <qpainter.h>
#include <qpalette.h>
#include <qfont.h>
#include <qwhatsthis.h>

#include "KLNfancywidgets.h"
#include "klearnnotes2-defs.h"

#include "globals.h"



// FancyButton and ActionWithIconSwitch

FancyButton::FancyButton(QString tekst, QWidget* parent)
  :QPushButton(tekst, parent),
   RandomObject()
{
  defaultcolor=paletteForegroundColor();
  colorchanged=false;
}

FancyButton::~FancyButton() {};

void FancyButton::drawButton ( QPainter * p)
{
  if(colorchanged)
    setPaletteForegroundColor (framecolor);
  else
    setPaletteForegroundColor (defaultcolor);
  QPushButton::drawButton(p);
  if(colorchanged)
    {
      p->setPen( QPen( framecolor,3 ) );
      p->drawRoundRect(rect ());
    }
}

void FancyButton::setColorTmp(QColor color, int time)
{
  framecolor=color;
  colorchanged=true;
  QTimer::singleShot( time, this,SLOT(resetColor()));
  update();
}
  
void FancyButton::resetColor()
{
  colorchanged=false;
  update();
}
//////////////////////////////

FancyQLCDNumber::FancyQLCDNumber( uint numDigits, QWidget * parent,
				  const char * name ) 
  :QLCDNumber(numDigits, parent, name ) 
{
  color=QColor(0,0,0);
}
 
FancyQLCDNumber::~FancyQLCDNumber() {};
  
void FancyQLCDNumber::setOK(bool isItOK)  // isItOK=> green, !isItOK=>red
{
  QPalette qp(palette());
  if(isItOK)
    {
      qp.setColor(QPalette::Normal,QColorGroup::Light,QColor(0,180,0));
      qp.setColor(QPalette::Normal,QColorGroup::Midlight,QColor(0,150,0));
      //qp->setColor(QPalette::Normal,QColorGroup::Button,QColor(0,255,0));
      qp.setColor(QPalette::Normal,QColorGroup::Mid,QColor(0,120,0));
      qp.setColor(QPalette::Normal,QColorGroup::Dark,QColor(0,90,0));
      qp.setColor(QPalette::Normal,QColorGroup::Shadow,QColor(0,60,0));
    }else{
      qp.setColor(QPalette::Normal,QColorGroup::Light,QColor(180,0,0));
      qp.setColor(QPalette::Normal,QColorGroup::Midlight,QColor(150,0,0));
      //qp->setColor(QPalette::Normal,QColorGroup::Button,QColor(255,0,0));
      qp.setColor(QPalette::Normal,QColorGroup::Mid,QColor(120,0,0));
      qp.setColor(QPalette::Normal,QColorGroup::Dark,QColor(90,0,0));
      qp.setColor(QPalette::Normal,QColorGroup::Shadow,QColor(60,0,0));
    }
  setPalette(qp);
  update();
}

/////////////////////////////////


ActionWithIconSwitch::ActionWithIconSwitch(QObject * parent, 
					   const char* pix1[],
					   const char* pix2[])
  :QAction(parent,0,true)
{
  iconSet1=new QIconSet(pix1);
  iconSet2=new QIconSet(pix2);
  connect( this, SIGNAL(toggled(bool)), this, SLOT( iconswitch(bool)) );
}


void ActionWithIconSwitch::iconswitch(bool state)
{
  if(state)
    setIconSet(*iconSet1);
  else
    setIconSet(*iconSet2);
}

QPixmap ActionWithIconSwitch::getSmallPixmap(int iconSetNo)
{ // this will be used to get small pixmap from the action to 
  // a corresponding combobox
  QPixmap answ;
  if(iconSetNo==1)
    answ=iconSet1->pixmap(QIconSet::Small,QIconSet::Normal);
  else
    answ=iconSet2->pixmap(QIconSet::Small,QIconSet::Normal);
  return answ;
}

//! a row with CDEFGAH buttons and their enablers
NameButtonsBox::NameButtonsBox(Exercise* exercise,
			       QWidget * parent, double scale)
  : RandomObjectsContainer(parent)
{
  //  debuginfo("NameButtonsBox() - start");
  mainHLayout  = new QHBoxLayout( this );
  for (int i =0; i<7; i++)
    {
      oneNameVLayout[i] = new QVBoxLayout( mainHLayout );
      oneNameVLayout[i]->setSpacing(0);

      nameButtons[i]=new 
	FancyButton(exercise,
		    new BaseMusicNote(BaseMusicNote::whiteNumberToTotalNumberInOctave(i)),
		    this);
      allQuestions.append(nameButtons[i]);
      QFont qf(nameButtons[i]->font());
      qf.setBold(true);
      qf.setStyleHint(QFont::SansSerif,
                      QFont::StyleStrategy(QFont::PreferAntialias |
                                           QFont::PreferQuality));
      qf.setPointSize(int(4 * scale));
      nameButtons[i]->setFont(qf);
      nameButtons[i]->setFixedWidth(int(5*scale));
      nameButtons[i]->setFixedHeight(int(5*scale));
      QWhatsThis::add(nameButtons[i], 
                      tr("<U><B>namebutton</B></U><UL>"
                         "<LI>at TEST : use it to give an answer - the "
                         "name of a displayed note "
                         "<LI>at STOP : click to highlight in the "
                         "<EM>staff</EM> all testnotes with the name "
                         "shown on the <EM>namebutton</EM></UL>"));
  //      connect(nameButtons[i], SIGNAL(nameActivatedSignal(BaseMusicNote* )),
      connect(nameButtons[i], SIGNAL(answerByName(BaseMusicNote* )),
	      this, SLOT(forwardNameActivatedSignal(BaseMusicNote* )));
      

      nameChecks[i]=new QCheckBox(this);
      nameChecks[i]->setChecked(true);
      QWhatsThis::add(nameChecks[i], 
                      tr("<U><B>name checkbox</B></U> :<BR>choose which "
                         "notes should be <EM>testnotes</EM><BR>"
                         "hint: choose Lesson %1, press a "
                         "<EM>namebutton</EM> (to see which notes it "
                         "corresponds to) and then a <EM>name "
                         "checkbox</EM> below it; play for a moment "
                         "with these <EM>checkboxes</EM> to get an idea "
                         "how they work").arg(defALLnotesEXEnum));
      connect(nameChecks[i],SIGNAL(stateChanged(int)),
	      this,SLOT(slotActiveNotesChanged(int)));

      extraKeyLabel[i]=new QLabel(QString("%1").arg(i+3),this);
      qf.setPointSize(int(0.5* scale));
      extraKeyLabel[i]->setFont(qf);
      
      oneNameVLayout[i]->addWidget(extraKeyLabel[i],0,Qt::AlignHCenter);
      oneNameVLayout[i]->addWidget(nameButtons[i],0,Qt::AlignHCenter);
      oneNameVLayout[i]->addWidget(nameChecks[i],0,Qt::AlignHCenter);
    }
  //  debuginfo("NameButtonsBox() - end");
}

FancyButton::FancyButton(Exercise* exercise,
			 BaseMusicNote* noteOfThisName, QWidget* parent)
  :QPushButton(noteOfThisName->stringName(), parent),
   RandomObject(exercise) 
{
  defaultcolor=paletteForegroundColor();
  colorchanged=false;
  noteOfButtonsName=noteOfThisName;
  connect(this, SIGNAL(clicked()), this, SLOT(emitMyName()));
}

void FancyButton::setName(BaseMusicNote * noteOfNewName)
{
  setText(noteOfNewName->stringName());
  noteOfButtonsName=noteOfNewName;
}

void FancyButton::updateNotesNames()
{
  setName(noteOfButtonsName);
}

void FancyButton::emitMyName()
{
  debuginfo("button emits it's name (BaseMusicNote)"); 
  emit answerByName(noteOfButtonsName);
}

BaseMusicNote* FancyButton::getNameNote()
{
  return noteOfButtonsName;
}


bool FancyButton::checkIfIsActive()
{
  bool check=true;
  if(!(myExercise->isActiveName(noteOfButtonsName)))
    check=false;

  RandomObject::active=check;
  setEnabled(check);
  return(check);
}




void NameButtonsBox::highlightName(BaseMusicNote* note,char color,
				   int time )
{
  //  debuginfo("highlighiting a button by name: "+note->stringName());
  for(int i=0;i<7;i++)
    if(((nameButtons[i]->getNameNote())->stringName())==
       (note->stringName()))
      if(color=='0')
	nameButtons[i]->resetColor();
      else
	if(nameButtons[i]->isActiveForTest())
	  nameButtons[i]->setColorTmp(defNiceBlue, time);
}
//void NameButtonsBox::setScale(double newScale);

bool NameButtonsBox::isActiveName(BaseMusicNote* noteToCheck)
{
  for(int i = 0; i <7 ; i++)
    if(BaseMusicNote::isEqualName(nameButtons[i]->getNameNote(),
				  noteToCheck))
      return nameChecks[i]->isOn();
  return false;
       //  if(! noteToCheck->isWhite())
       //    {
       //      return false;
       //    }
       //  int numOfWhite=
       //    BaseMusicNote::pitchToWhiteNumberInOctave(noteToCheck->pitch());
       //  return nameChecks[numOfWhite]->isOn();
}

void NameButtonsBox::slotActiveNotesChanged(int)
{
  debuginfo("emiting activeNotesChanged()");
  emit activeNotesChanged();
}

void NameButtonsBox::forwardNameActivatedSignal(BaseMusicNote* 
						noteOfActivatedName)
{
  // the external stuff hass access only to NameButtonsBox signals;
  // signals emited by nameButtons[] have to be re-emited
  debuginfo(QString("NameButtonsBox: re-emiting FancyButton::answerByName(%1)"
		    ).arg(noteOfActivatedName->stringName()));
  emit  answerByName(noteOfActivatedName);
}

void NameButtonsBox::updateNotesNames()
{
  for(int i=0;i<7;i++)
    nameButtons[i]->updateNotesNames();
}
