/***************************************************************************
 ** $Id: KLNfancywidgets.h,v 1.8 2004/05/13 02:57:19 wiecko Exp $
                          klearnnotes2
                          KLNfancywidgets.h -  description
                             -------------------
    begin                : Wed Oct  8 15:32:54 CEST 2003
    copyright            : (C) 2003 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KLN_FANCY_WIDGETS_H
#define KLN_FANCY_WIDGETS_H


#include <qaction.h>
#include <qpushbutton.h>
#include <qlcdnumber.h>
#include "musicwidget.h"
#include <qlabel.h>
#include <qcheckbox.h>
#include <qlayout.h> 
#include "randomobject.h"

class QObject;
class QPixmap;
class QColor;

//! A toggle QAction (menu- and toolbar item) with two icons: for on and off states
/*! Qt toggle action can change its icon in toolbar button, but not in menu item
  (in menu the same icon is just "pressed down").

  This class defines a QAction which uses two different icons in menus. It
  is used for sound on/off, voice recognition on/off, treble/bass clef (in 
  off state it uses empty pixmap) and so on.
*/
class ActionWithIconSwitch:public QAction
{
  Q_OBJECT
    public:
  //! creator
  /*! pix0 is a xpm pixmap for on state, pix1 for off state. */
  ActionWithIconSwitch(QObject * parent, 
		       const char* pix0[],
		       const char* pix1[]);
  //! Returns the on or off pixmap.
  /*! Used to have the same icons in comboboxes (e.g. for clef choice). */
  QPixmap getSmallPixmap(int iconSetNo);

 private:
  QIconSet* iconSet1 ;
  QIconSet* iconSet2 ;
  private slots:
    //! Changes state (icon) of the action to on or off.
    void iconswitch(bool state);
};




//! A QPushButton with possibility to change frame color.
class FancyButton : public QPushButton, public RandomObject
{
  Q_OBJECT
    public:
  //! OBSOLETE!
  /*! note, that after constructing FancyButton this way some properties
    are not initialized!!!! */
  FancyButton(QString tekst, QWidget* parent);
  FancyButton(Exercise* exercise,
	      BaseMusicNote* noteOfThisName, QWidget* parent);
  virtual ~FancyButton();
  
  public slots:
    //! sets noteOfButtonsName , and based on this button text
    void setName(BaseMusicNote* newNoteOfButtonsName);
  
  //! sets colorchanged to false
  void resetColor();
  //! sets colorchanged to true and QTimer::singleShot()s resetColor()
  void setColorTmp(QColor color, int time);
  //! Overridden QPushButton::drawButton()
  /*! Runs QPushButton::drawButton() and if the button should be highlighted
    [ colorchanged == true ] draws a blue frame.
  */
  void drawButton ( QPainter * );
  //! when the button is clicked, this is used to emit it's name
  void emitMyName();

  BaseMusicNote* getNameNote();

  virtual bool checkIfIsActive();

  virtual void updateNotesNames();

 signals:
  void answerByName(BaseMusicNote* noteOfActivatedName);
    //nameActivatedSignal(BaseMusicNote* noteOfActivatedName);
 private:
  //! A color stored for resetColor(). ???? is it used at all?
  QColor defaultcolor;
  //! A color for highlighted frame.
  QColor framecolor;
  //! if true - highlight frame is drawn, if false nothing happens
  bool colorchanged;

  //! example of a note of this name
  /*! We do not store numOfNameInOctave because in future there may be
    needed distiction between, say C# and Db (and for both numOfNameInOctave
    is 1).

    To be used only when notation changes.
  */
  BaseMusicNote* noteOfButtonsName;
};

//! a box holding 7 FancyButtons with names of notes (CDEFGAB)
class NameButtonsBox://public QFrame,
		     public RandomObjectsContainer
{
  Q_OBJECT
 public:
  NameButtonsBox(Exercise* exercise, QWidget * parent,double scale);

 signals:
  //  void nameActivated(BaseMusicNote* noteOfActivatedName);
  void activeNotesChanged();

  public slots:
    virtual void highlightName(BaseMusicNote*,char color ='B', 
			       int time = TIMEOFHIGHLIGHT);
  //  virtual void setScale(double newScale);
  virtual void updateNotesNames();
 public:
  //! should notes of this name be considered active (==a testnote)?
  /*! MusicWidgets have features which choose active notes. For example
    button box has checkboxes for different note names, staff has range
    sliders.

    With this method a note can ask: "hi, I'm C1. Should I be considered 
    active?".

    The note is active only if it is active according to all Exercise's 
    widgets.
  */
  virtual bool isActiveName(BaseMusicNote*);
 protected:
  //! these display numbers 3..9, but this additional comp.kbd input is not used for now! [FIXME]
  QLabel*  extraKeyLabel[7];
  QButtonGroup* nameButtonsGroup;
  FancyButton* nameButtons[7];
  QCheckBox* nameChecks[7];
  int graphScale;


  QBoxLayout* mainHLayout;
  //! vertical layouts for a button and its checkbox
  QBoxLayout* oneNameVLayout[7];
  protected slots:
    void slotActiveNotesChanged(int);
  void forwardNameActivatedSignal(BaseMusicNote* noteOfActivatedName);
};

//! Numeric display with easy frame color changes.
class FancyQLCDNumber : public QLCDNumber
{
  Q_OBJECT
    public:
  FancyQLCDNumber( uint numDigits, QWidget * parent = 0,
		   const char * name = 0 ) ;
  ~FancyQLCDNumber();
  
  public slots:
    //! setOK() changes frame color to green; setOK(false) changes it to red.
    void setOK(bool isItOK=true);
 private:
  //! ????? not used? I dunno. Probably left from old version and could be deleted.
  QColor color;
};

#endif// KLN_FANCY_WIDGETS_H
