/***************************************************************************
 ** $Id: customlevel.cpp,v 1.4 2004/01/22 04:57:20 wiecko Exp $
                          klearnnotes2
                          customlevel.cpp -  description
                             -------------------
    begin                : Wed Oct  8 15:32:54 CEST 2003
    copyright            : (C) 2003 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "customlevel.h"

#include <qvariant.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include "customlevel.ui.h"

CustomLevel::CustomLevel( QPixmap pmapForWhatsThisButton, QWidget* parent,
			  int val2, int val7,
			  int last2, int last7,
			  const char* name, bool modal, WFlags fl )
  : QDialog( parent, name, modal, fl )
{
  if ( !name ) setName( "customLevel" );

  lastSpeed2=last2;//last custom speed for 2 namebuttons
  lastSpeed7=last7;//last custom speed for 7 namebuttons

  resize( 700, 234 ); 
  setCaption( tr( "setup custom level" ) );

  CustomLevel* privateLayoutWidget=this;

  QVBoxLayout* layout11 = new QVBoxLayout( privateLayoutWidget, 20, 6,
					   "Layout11"); 

  // ************ GRID SETUP
  QGridLayout* layout1 = new QGridLayout( 0, 1, 1, 0, 10, "Layout1"); 

  QLabel* TextLabel0 = new QLabel( privateLayoutWidget, "TextLabel0" );
  TextLabel0->setText( tr( "for the following number of "
			   "active namebuttons..." ) );
  layout1->addWidget( TextLabel0, 0, 0 );
  QLabel* TextLabel1 = new QLabel( privateLayoutWidget, "TextLabel1" );
  TextLabel1->setText( tr( "... custom speed goal will be:" ) );
  TextLabel1->setAlignment( QLabel::AlignVCenter |QLabel::AlignRight );
  layout1->addWidget( TextLabel1, 1, 0 );

  QLabel* TextLabel27[6]; // text labels 2-7 with numbers of buttons	
  for(int i=0;i<6;i++)
    {
      TextLabel27[i] = new QLabel( privateLayoutWidget );
      QFont tmpfont(  TextLabel27[i]->font() );
      tmpfont.setBold( true );
      TextLabel27[i]->setFont( tmpfont ); 
      TextLabel27[i]->setFrameShape( QLabel::Box );
      TextLabel27[i]->setFrameShadow( QLabel::Raised );
      TextLabel27[i]->setText( QString("%1").arg(i+2) );
      TextLabel27[i]->setAlignment( QLabel::AlignVCenter |QLabel::AlignRight);
      layout1->addWidget( TextLabel27[i], 0, i+1 );
    }
  spinBox_speed2 = new QSpinBox( privateLayoutWidget, "spinBox_speed2" );
  spinBox_speed2->setMaxValue( 700 );
  spinBox_speed2->setMinValue( 50 );
  spinBox_speed2->setValue( val2 );
  QToolTip::add( spinBox_speed2, tr( "speed goal for 2 namebuttons" ) );
  QWhatsThis::add( spinBox_speed2 , tr("choose <B><U>speed "
				       "goal </B></U> for 2 namebuttons active"
				       "(in answers per 10 minutes); "
				       "this is in range "
				       "50 (very easy) to 700 (impossible with"
				       " mouse, maybe achievable "
				       "with keyboard)"));
  layout1->addWidget( spinBox_speed2, 1, 1 );

  comments2 = new QLabel( privateLayoutWidget, "comments2" );
  comments2->setText( tr( "Easy" ) );
  comments2->setAlignment( QLabel::AlignRight );
  layout1->addMultiCellWidget( comments2, 2, 2, 0, 1 );


  spinBox_speed7 = new QSpinBox( privateLayoutWidget, "spinBox_speed7" );
  spinBox_speed7->setMaxValue( 700 );
  spinBox_speed7->setMinValue( 50 );
  spinBox_speed7->setValue( val7 );
  QToolTip::add( spinBox_speed7, tr( "speed goal for 7 namebuttons" ) );
  QWhatsThis::add( spinBox_speed7 , tr("choose <B><U>speed "
				       "goal </B></U> for 7 namebuttons active (in answers per 10 minutes); "
				       "this is in range "
				       "50 (very easy) to 700 (note, that already 400 impossible with mouse, maybe achievable "
				       "with keyboard)"));
  layout1->addWidget( spinBox_speed7, 1, 6 );

  comments7 = new QLabel( privateLayoutWidget, "comments7" );
  comments7->setText( tr( "Easy" ) );
  comments7->setAlignment(QLabel::AlignRight );
  layout1->addMultiCellWidget( comments7, 2, 2, 5, 6 );

  for(int i=0;i<4;i++)
    {
      speed36[i] = new QLabel( privateLayoutWidget );
      speed36[i]->setFrameShape( QLabel::Box );
      speed36[i]->setFrameShadow( QLabel::Raised );
      speed36[i]->setText( "500" );
      speed36[i]->setAlignment( QLabel::AlignVCenter |QLabel::AlignRight );
      QToolTip::add( speed36[i],tr("speed for %1 namebuttons "
				   "active").arg(i+3));
      QWhatsThis::add( speed36[i], tr("speed for %1 namebuttons "
				      "active; "
				      "<BR>this is interpolated from speeds for 2 and 7 "
				      "namebuttons active").arg(i+3));
      layout1->addWidget( speed36[i], 1, 2+i );
    }

  labelComments = new QLabel( privateLayoutWidget, "labelComments" );
  labelComments->setText( "" );
  layout1->addMultiCellWidget( labelComments, 3, 3, 0, 6 );


  layout11->addLayout( layout1 );



  QHBoxLayout* layout10 = new QHBoxLayout( 0, 0, 6, "Layout10"); 

  GroupBox2 = new QGroupBox( privateLayoutWidget, "GroupBox2" );
  GroupBox2->setFrameShape( QGroupBox::Box );
  GroupBox2->setTitle( tr( "Use defaults for:" ) );

  pushButton_goal0 = new QPushButton( GroupBox2, "pushButton_goal0" );
  pushButton_goal0->setGeometry( QRect( 11, 21, 92, 24 ) ); 
  pushButton_goal0->setCursor( QCursor( 13 ) );
  pushButton_goal0->setText( tr( "&Prentice" ) );
  QWhatsThis::add( pushButton_goal0 , tr("click this button to copy preset "
					 "<U><B>easy</B></U> speed "
					 "goals for 2 and 7 namebuttons active to the fields above; you can then modify these "
					 "values"));

  pushButton_goal1 = new QPushButton( GroupBox2, "pushButton_goal1" );
  pushButton_goal1->setGeometry( QRect( 109, 21, 92, 24 ) ); 
  pushButton_goal1->setCursor( QCursor( 13 ) );
  pushButton_goal1->setText( tr( "&Journeyman" ) );
  QWhatsThis::add( pushButton_goal1 , tr("click this button to copy preset "
					 "<U><B>intermediate</B></U> speed "
					 "goals for 2 and 7 namebuttons active to the fields above; you can then modify these "
					 "values"));

  pushButton_goal2 = new QPushButton( GroupBox2, "pushButton_goal2" );
  pushButton_goal2->setGeometry( QRect( 207, 21, 92, 24 ) ); 
  pushButton_goal2->setCursor( QCursor( 13 ) );
  pushButton_goal2->setText( tr( "&Wizard" ) );
  QWhatsThis::add( pushButton_goal2 , tr("click this button to copy preset "
					 "<U><B>hard</B></U> speed "
					 "goals for 2 and 7 namebuttons active to the fields above; you can then modify these "
					 "values"));

  pushButton_lastCustom = new QPushButton( GroupBox2, "pushButton_lastCustom" );
  pushButton_lastCustom->setGeometry( QRect( 305, 21, 92, 24 ) ); 
  pushButton_lastCustom->setCursor( QCursor( 13 ) );
  pushButton_lastCustom->setText( tr( "&last custom" ) );
  QWhatsThis::add( pushButton_lastCustom , tr("click this button to restore last "
					      "custom speed goals set by you"));



  layout10->addWidget( GroupBox2 );
  QVBoxLayout* layout6 = new QVBoxLayout( 0, 0, 6, "Layout6"); 

  empty_label = new QLabel( privateLayoutWidget, "empty_label" );
  empty_label->setText( "" );
  layout6->addWidget( empty_label );

  QHBoxLayout* layout5 = new QHBoxLayout( 0, 0, 6, "Layout5"); 

  pushButton_what = new QPushButton( privateLayoutWidget, "pushButton_what" );
  pushButton_what->setPixmap(pmapForWhatsThisButton);
  QToolTip::add(pushButton_what,tr("What's this?"));
  layout5->addWidget( pushButton_what ,0,Qt::AlignLeft);


  pushButton_ok = new QPushButton( privateLayoutWidget, "pushButton_ok" );
  pushButton_ok->setText( tr( "OK" ) );
  pushButton_ok->setDefault( TRUE );
  layout5->addWidget( pushButton_ok );

  pushButton_cancel = new QPushButton( privateLayoutWidget, "pushButton_cancel" );
  pushButton_cancel->setText( tr( "Cancel" ) );
  layout5->addWidget( pushButton_cancel );
  layout6->addLayout( layout5 );
  layout10->addLayout( layout6 );
  layout11->addLayout( layout10 );

  // signals and slots connections
  connect( pushButton_goal0, SIGNAL( clicked() ), this, SLOT( pushButton_goal0_clicked() ) );
  connect( pushButton_goal1, SIGNAL( clicked() ), this, SLOT( pushButton_goal1_clicked() ) );
  connect( pushButton_goal2, SIGNAL( clicked() ), this, SLOT( pushButton_goal2_clicked() ) );
  connect( pushButton_lastCustom, SIGNAL( clicked() ), this, SLOT( pushButton_lastCustom_clicked() ) );
  connect( pushButton_what, SIGNAL( clicked() ), this, SLOT( pushButton_what_clicked() ) );
  connect( pushButton_ok, SIGNAL( clicked() ), this, SLOT( pushButton_OK_clicked() ) );
  connect( pushButton_cancel, SIGNAL( clicked() ), this, SLOT( pushButton_cancel_clicked()));
  connect( spinBox_speed7, SIGNAL( valueChanged(int) ), this, 
	   SLOT( setupSpeeds36(int) ) );
  connect( spinBox_speed2, SIGNAL( valueChanged(int) ), this, 
	   SLOT( setupSpeeds36(int) ) );
  setupSpeeds36();
}

CustomLevel::~CustomLevel(){}

