/***************************************************************************
 ** $Id: exe-piano_name.cpp,v 1.4 2004/05/13 02:57:20 wiecko Exp $
                          klearnnotes2
                          exe-piano_name.cpp -  description
                             -------------------
    begin                : Sat Apr  3 01:13:45 CEST 2004
    copyright            : (C) 2004 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

#include "exe-piano_name.h"

#include "basemusicnote.h"
#include<qcombobox.h>
#include<qwhatsthis.h>
#include<qtooltip.h>
#include"pics.h"

ExercisePianoToName::ExercisePianoToName(KLearnNotes2* kln2_)
  :Exercise(kln2_)
{
  kln2=kln2_;
  //  debuginfo("ExercisePianoToName() - start");
  myLayout=new QVBoxLayout( this, 10, 10);
  myLayout->addStretch(10);
  pianoKeyboard=new PianoKeyboard(this,this,48, 41, 500, kln2);
  myLayout->addWidget(pianoKeyboard);
  myLayout->addStretch(10);
  nameButtonsBox=new NameButtonsBox(this,this, 10);
  myLayout->addWidget(nameButtonsBox);
  myLayout->addStretch(10);

  toolbar=new QToolBar(kln2,"ExercisePianoToName");
  toolbar->setLabel( tr( "Exercise: Piano key to Note name" ) );
  toolbar->setNewLine ( true );
  toolbar->hide();

  bStartStop = new QPushButton( tr("&Start"), toolbar );
  QToolTip::add( bStartStop,tr("Start/stop a test","tooltip"));


  QLabel* labi = new QLabel(tr("&L:","_L_esson"),toolbar) ;
  QComboBox* combo_lesson = new QComboBox( FALSE, toolbar);
  combo_lesson->setFixedWidth(280);
  labi->setBuddy(combo_lesson);
  QWhatsThis::add(combo_lesson, 
                  tr("<B><U>choose Lesson</B></U> from the list;<BR>"
                     "this chooses a predefined set of"
                     " <EM>testnotes</EM>"));
  QToolTip::add( combo_lesson,tr("choose Lesson"));
  QAction* aNextLesson=new QAction(tr("Next lesson",
				      "menu txt in the statusbar"),
				   QPixmap( (const char**)next_xpm ),
				   tr("Next &lesson"), 
				   QKeySequence(tr("ALT+N","_N_ext lesson")), this,
				   "lesson++" );
  aNextLesson->addTo(toolbar);
  aNextLesson->setToolTip(tr("next Lesson","tooltip"));

  connect(bStartStop,SIGNAL(clicked()),
	  this,SLOT(startStop()));

  connect(nameButtonsBox,SIGNAL(activeNotesChanged()),
	  pianoKeyboard,SLOT(setupActiveQuestions()));
  connect(nameButtonsBox,SIGNAL(activeNotesChanged()),
	  nameButtonsBox,SLOT(setupActiveQuestions()));
  connect(pianoKeyboard,SIGNAL(activeNotesChanged()),
	  pianoKeyboard,SLOT(setupActiveQuestions()));
  connect(pianoKeyboard,SIGNAL(activeNotesChanged()),
	  nameButtonsBox,SLOT(setupActiveQuestions()));

  connect( kln2, SIGNAL(notationChanged()),
	   pianoKeyboard,SLOT(updateNotesNames()));
  connect( kln2, SIGNAL(notationChanged()),
	   nameButtonsBox,SLOT(updateNotesNames()));

  pianoKeyboard->setupActiveQuestions();
  nameButtonsBox->setupActiveQuestions();
  //  hide();
  //  debuginfo("ExercisePianoToName() - end");
}

ExercisePianoToName::~ExercisePianoToName()
{
  delete pianoKeyboard;
  delete nameButtonsBox;
}

bool ExercisePianoToName::isActiveName(BaseMusicNote* note)
{
  // note, that piano keyboard does not select active notes by name
  // - we check here only nameButtonsBox:
  if(nameButtonsBox->isActiveName(note))
    return true;
  return false;
}
bool ExercisePianoToName::isActivePitch(BaseMusicNote* note)
{
  // note, that nameButtonsBox does not distinguish pitches - we check
  // here only the pianoKeyboard :
  if(pianoKeyboard->isActivePitch(note))
    return true;
  return false;
}

bool ExercisePianoToName::isActiveNote(BaseMusicNote* note)
{
  if((nameButtonsBox->isActiveName(note)
      )&&(pianoKeyboard->isActivePitch(note)))
    {
      //    debuginfo(QString("ExercisePianoToName::isActiveNote %1 %2 - TRUE"
      //		).arg(note->stringName(true)).arg(note->pitch()));
      return true;
    }
  //  debuginfo(QString("ExercisePianoToName::isActiveNote %1 %2 - FALSE"
  //		    ).arg(note->stringName(true)).arg(note->pitch()));
  return false;
}

void ExercisePianoToName::show()
{
  QFrame::show();
  toolbar->show();
  connectIdle();
}

void ExercisePianoToName::connectIdle()
{
  connect(kln2,SIGNAL(keyPressed(BaseMusicNote *)),
	  nameButtonsBox,SLOT(highlightName(BaseMusicNote *)));
  connect(kln2,SIGNAL(keyPressed(BaseMusicNote *)),
	  pianoKeyboard,SLOT(highlightAllOfName(BaseMusicNote*)));

  connect(pianoKeyboard,SIGNAL(answerByNote(BaseMusicNote*)),
	  nameButtonsBox,SLOT(highlightName(BaseMusicNote*)));
  // connecting here answerByPitch, to avoid mulitple triggering:
  connect(pianoKeyboard,SIGNAL(answerByPitch(BaseMusicNote*)),
	  pianoKeyboard,SLOT(highlightPitch(BaseMusicNote*)));
  connect(nameButtonsBox,SIGNAL(answerByName(BaseMusicNote*)),
	  pianoKeyboard,SLOT(highlightAllOfName(BaseMusicNote*)));
  connect(nameButtonsBox,SIGNAL(answerByName(BaseMusicNote*)),
	  nameButtonsBox,SLOT(highlightName(BaseMusicNote*)));
}

void ExercisePianoToName::disconnectIdle()
{
  disconnect(kln2,SIGNAL(keyPressed(BaseMusicNote *)),
	     nameButtonsBox,SLOT(highlightName(BaseMusicNote *)));
  disconnect(kln2,SIGNAL(keyPressed(BaseMusicNote *)),
	     pianoKeyboard,SLOT(highlightAllOfName(BaseMusicNote*)));

  disconnect(pianoKeyboard,SIGNAL(answerByNote(BaseMusicNote*)),
	     nameButtonsBox,SLOT(highlightName(BaseMusicNote*)));
  // connecting here answerByPitch, to avoid mulitple triggering:
  disconnect(pianoKeyboard,SIGNAL(answerByPitch(BaseMusicNote*)),
	     pianoKeyboard,SLOT(highlightPitch(BaseMusicNote*)));
  disconnect(nameButtonsBox,SIGNAL(answerByName(BaseMusicNote*)),
	     pianoKeyboard,SLOT(highlightAllOfName(BaseMusicNote*)));
  disconnect(nameButtonsBox,SIGNAL(answerByName(BaseMusicNote*)),
	     nameButtonsBox,SLOT(highlightName(BaseMusicNote*)));

}

void ExercisePianoToName::hide()
{
  stop();
  disconnectIdle();
  QFrame::hide();
  toolbar->hide();
}

void ExercisePianoToName::startStop()
{
  if (isStarted())
    {
      bStartStop->setText(tr("&Start"));
      stop();
      return;
    }
  bStartStop->setText(tr("&Stop"));
  start();
}


void ExercisePianoToName::start()
{
  if(! isStarted())
    {
      debuginfo("ExercisePianoToName::start()");
      Exercise::start();// this emits starting()
      disconnectIdle();
      connect(kln2,SIGNAL(keyPressed(BaseMusicNote *)),
	      pianoKeyboard,SLOT(checkAnswerByName(BaseMusicNote*)));
      connect(nameButtonsBox,SIGNAL(answerByName(BaseMusicNote*)),
	      pianoKeyboard,SLOT(checkAnswerByName(BaseMusicNote*)));
      connect(pianoKeyboard,SIGNAL(resetStatistics()),
	      kln2,SLOT(resetStatistics()));
      connect(pianoKeyboard,SIGNAL(answerWasRight(BaseMusicNote*,int,bool)),
	      kln2,SLOT(answerWasRight(BaseMusicNote*,int,bool)));
      connect(pianoKeyboard,SIGNAL(comment(QString)),
	      kln2,SLOT(comment(QString)));
      connect(pianoKeyboard,SIGNAL(stopped()),
	      this, SLOT(stop()));
      pianoKeyboard->startTest(10);
    }
}

void ExercisePianoToName::stop()
{
  if(isStarted())
    {
      debuginfo("ExercisePianoToName::stop()");
      Exercise::stop();//this emits stopping()
      pianoKeyboard->stopTest();
      disconnect(kln2,SIGNAL(keyPressed(BaseMusicNote *)),
		 pianoKeyboard,SLOT(checkAnswerByName(BaseMusicNote*)));
      disconnect(nameButtonsBox,SIGNAL(answerByName(BaseMusicNote*)),
		 pianoKeyboard,SLOT(checkAnswerByName(BaseMusicNote*)));
      disconnect(pianoKeyboard,SIGNAL(resetStatistics()),
		 kln2,SLOT(resetStatistics(int)));
      disconnect(pianoKeyboard,SIGNAL(answerWasRight(BaseMusicNote*,int,bool)),
		 kln2,SLOT(answerWasRight(BaseMusicNote*,int,bool)));
      disconnect(pianoKeyboard,SIGNAL(comment(QString)),
		 kln2,SLOT(comment(QString, QColor)));
      connectIdle();
    }
}

ExerciseNameToPiano::ExerciseNameToPiano(KLearnNotes2* kln2_)
  :ExercisePianoToName(kln2_)
{
  // constructor of ExercisePianoToName sets up some basic things;
  // we have to rearange them here:

  delete myLayout;

  myLayout=new QVBoxLayout( this, 10, 10);
  myLayout->addStretch(10);
  myLayout->addWidget(nameButtonsBox);
  myLayout->addStretch(10);
  myLayout->addWidget(pianoKeyboard);
  myLayout->addStretch(10);

  toolbar->setLabel( tr( "Exercise:  Note name to Piano key" ) );
}

void ExerciseNameToPiano::start()
{
  if(! isStarted())
    {
      debuginfo("ExerciseNameToPiano::start()");
      Exercise::start();// this emits starting()
      disconnectIdle();
      //      connect(kln2,SIGNAL(keyPressed(BaseMusicNote *)),
      //	      pianoKeyboard,SLOT(checkAnswerByName(BaseMusicNote*)));
      connect(pianoKeyboard,SIGNAL(answerByName(BaseMusicNote*)),
	      nameButtonsBox,SLOT(checkAnswerByName(BaseMusicNote*)));
      connect(nameButtonsBox,SIGNAL(resetStatistics()),
	      kln2,SLOT(resetStatistics()));
      connect(nameButtonsBox,SIGNAL(answerWasRight(BaseMusicNote*,int,bool)),
	      kln2,SLOT(answerWasRight(BaseMusicNote*,int,bool)));
      connect(nameButtonsBox,SIGNAL(comment(QString)),
	      kln2,SLOT(comment(QString)));
      connect(nameButtonsBox,SIGNAL(stopped()),
	      this, SLOT(stop()));
      nameButtonsBox->startTest(10);
    }
}

void ExerciseNameToPiano::stop()
{
  if(isStarted())
    {
      debuginfo("ExerciseNameToPiano::stop()");
      Exercise::stop();//this emits stopping()
      nameButtonsBox->stopTest();
      //      disconnect(kln2,SIGNAL(keyPressed(BaseMusicNote *)),
      //	 pianoKeyboard,SLOT(checkAnswerByName(BaseMusicNote*)));
      disconnect(pianoKeyboard,SIGNAL(answerByName(BaseMusicNote*)),
		 nameButtonsBox,SLOT(checkAnswerByName(BaseMusicNote*)));
      disconnect(nameButtonsBox,SIGNAL(resetStatistics()),
		 kln2,SLOT(resetStatistics(int)));
      disconnect(nameButtonsBox,SIGNAL(answerWasRight(BaseMusicNote*,int,bool)),
		 kln2,SLOT(answerWasRight(BaseMusicNote*,int,bool)));
      disconnect(nameButtonsBox,SIGNAL(comment(QString)),
		 kln2,SLOT(comment(QString, QColor)));
      connectIdle();
    }
}
