/***************************************************************************
 ** $Id: exercise.h,v 1.4 2004/05/13 02:57:20 wiecko Exp $
                          klearnnotes2
                          exercise.h -  description
                             -------------------
    begin                : Tue Mar 30 03:17:17 CEST 2004
    copyright            : (C) 2004 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

#ifndef KLN_EXERCISE_H
#define KLN_EXERCISE_H
#include <qaction.h>
#include <qtoolbar.h>
#include <qframe.h>
#include "klearnnotes2.h"
#include "basemusicnote.h"


//! class which sets up a specific exercise
/*! This class is a kind of in-the-middle interface.

From main class KLearnNotes2 window point of view class Exercise sets up 
widgets for specific exercise - class KLearnNotes2  does not have to deal
with details of specific exercise directly. From exercise widgets point 
of view class Exercise provides an interface for checking out which 
notes should be active (due to preset lessons or settings in other
widgets), setting up interaction with keyboard etc.

There are three points of view on a structure of the code:
* #. what is a parent of a widget
   - this only sets how things are displayed; here only QFrames should be
   "parented" by class KLearnNotes2 central widget
* #. what is a property of which class:

class KLearnNotes2 has:
* - central widget
* - results display
* - comments text box
* - menus
* - a set of class Exercise -s

class Exercise has:
* - a menu class QAction (parented by class KLearnNotes2)
* - a QToolbar (parented by class KLearnNotes2)
* - a question QFrame (parented by cental widget of class KLearnNotes2)
* - an answer QFrame (parented by cental widget of class KLearnNotes2)

* #. which signals are connected to which slots
* - class KLearnNotes2 signals from computer keyboard, voice and midi
    keyboard inputs should trigger question's and answer's highlight() or
    checkanswer();
* - question's signals due to wrong/right answer should update results display
* - starting/stopping a test should enable/disable parts of main KLN2
* - in the end of test some comments should be displayed

within class Exercise:
* - question's and answer's active notes settings should trigger each other
* - question's and answer's clicking/highlight should trigger each other
* - some reaction on user clicking on the toolbar (lesson, clef)
*/
class Exercise: public QFrame//QObject
{
  Q_OBJECT
    public:
  //! sets up question/answer widgets, toolbar, qaction
  /*! Note, that signals are not connected until the exercise is activated. */
  Exercise(KLearnNotes2* kln2_);
  virtual ~Exercise();

  //! checks if note's name should be considered active
  /*! For example checkboxes at nameButtonsBox desactivate some notes' names.

  How should it be used?
  * - when a RandomObjectsContainer is created all it's RandomObjects should
      get the parent Exercise as a parameter
  * - when any RandomObjectContainer is informed by a singal that it should
      check what is active, it goes through all it's randomObjects and asks
      each if it is active; if true it is added to set of activeObjects
  * - when a RandomObject is asked if it is active, it should run Exercise's
      isActiveName/Pitch/Note
      (for example, a nameButton will run isActiveName; a keyboardKey will
      run isActiveName too for each of its notes and then isActivePitch (to
      see if a note is not ruled out by some range) and get desactivated if 
      none of the names is active or the pitch is inactive)
  * - the Exercise (methods below!) should ask (forward question to) each
      of its RandomObjectsContainer-s if the Name/Pitch/Note should be
      considered active; if any of the answers is "no", this returns false
      to the RandomObject
*/
  virtual bool isActiveName(BaseMusicNote*);
  //! checks if note's pitch should be considered active
  virtual bool isActivePitch(BaseMusicNote*);
  //! checks if an exact specific note should be considered active
  virtual bool isActiveNote(BaseMusicNote*);

    //! activates the exercise
    /*! shows question/answer widgets and the toolbar;connects slots/signals */
    virtual void show();
  //! desactivate the exercise
  /*! hides question/answer widgets and the toolbar;disconnects signals */
  virtual void hide();

  //! returns true if a test is in progress
  virtual bool isStarted();

  public slots:
    //! starts a test
    virtual void start();
  //! stops current test
  virtual void stop();

 signals:
    void starting();
  void stopping();

 protected:
  KLearnNotes2* kln2;
  QAction* menuAction;
  QToolBar* toolbar;
  QPushButton* bStartStop;
  QFrame* question; // should be MusicWidget* !
  QFrame* answer;
  bool started;
};

#endif // KLN_EXERCISE_H
