/***************************************************************************
 ** $Id: globals.h,v 1.10 2004/05/13 02:57:20 wiecko Exp $
                          klearnnotes2
                          globals.h  -  description
                             -------------------
    begin                : Thu Oct 23 21:36:44 CEST 2003
    copyright            : (C) 2003 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qstring.h>
#include <stdio.h> // printf
///////////// GLOBALS
//! should we write any output to stderr and stdout?
/*! quiet == true turns off output functions: debuginfo() etc. */
extern bool quiet;

//! KLNmath_min(a,b) returns smaller of two values (minimum)
/*! Just a standard definition. 

More clear would be to either include some math.h or to define this
as MIN(a,b). Neither works. On some systems math libs define MIN 
under another name. On others MIN(a,b) defining header is included
even if not included here explicitly. Thus, KLNmath_min(a,b) 
defines a minimum function which does what it is supposed to do and
does not clash with any other included defines.
*/
#define KLNmath_min(a,b) ((a<b)?(a):(b))

//! KLNmath_max(a,b) returns bigger of two values (maximum)
/*! Just a standard definition. 

See KLNmath_min() for some comments.
*/
#define KLNmath_max(a,b) ((a>b)?(a):(b))

//! KLNmath_power(double base, int exponent) returns base to power exponent
double KLNmath_power(double base,int exponent);

//! KLNmath_floor(value) returns an the greatest integer smaller than `value'
/*! Just a standard definition. 

See KLNmath_min() for some comments.
*/
int KLNmath_floor(double);

//! sleep without freeze
/*! Sleeps for usec microseconds. But it allows processing qEvents every
  300microseconds.

  Therefore, execution of the code following KLN_usleep() is delayed,
  but meanwhile program does not freeze - widgets are updated and user
  can interact with them.

  Other way to do the same would be to extract the rest of the code to
  another function and to run it with QTimer::singleShot. This is 
  the way it is done few times here, but it makes the code less clear
  and should be avoided.
*/
void KLN_usleep(unsigned long usec);

//! scale for all graphical objects (buttons, staff, clefs, notes)
extern int SC;

//! prints `"DEBUG: " +tekst' to stdout
/*! Generally so far this function was used to print some debug info whenever
  a new part of code was written.

  Sometimes it informs user about unfulfilled conditions (`couldn't initialize 
  midi', `igain not supported' etc.). In such cases usually the same info is also
  displayed for a moment in status bar.


  In future kln2 should support a --quiet command line option which should 
  turn this one off.
*/
void debuginfo(QString tekst);  // print debug info to stdout
//! prints `"KLearnNotes2: " + text' to stderr
/*! Informs user about non-critical errors.

In future kln2 should support a --quiet command line option which should 
turn this one off.
*/
void printerror(QString tekst); // print error to stderr

//! prints both variable name and its value to stdout
/*! Called by DEBUG_VAR(var_name) gives output 'KLearnNotes2: var_name=var_value'.
  VERY convenient for debugging. 
*/
#define DEBUG_VAR(x) if(!quiet) printf("%s",(QString("KLearnNotes2: ")+QString(#x)+" = "+QString("%1\n").arg(x)).latin1());
//#define DEBUG_VAR(x) cout <<"KLearnNotes2: "<< #x <<" = " << x << endl



//! just gets rid of "unused variable" warning
#define USENOTE(x) BaseMusicNote* fldadjfNOTE=x;x=fldadjfNOTE
#define USEBOOL(x) bool fldadjfBOOL=x;x=fldadjfBOOL
