/***************************************************************************
 ** $Id: imageitem.h,v 1.2 2004/01/28 04:22:48 wiecko Exp $
                          klearnnotes2
                          imageitem.h  -  description
                             -------------------
    begin                : Wed Oct  8 15:32:54 CEST 2003
    copyright            : (C) 2003 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qcanvas.h>
#include <qimage.h>
#include <qpainter.h>
#include <qpixmap.h>

//! a class for displaying clefs;
/*! It is just a QCanvasRectangle (it has width, height, can be moved etc.),
  but drawShape() is overriden: instead of showing a solid color rectangle
  a pixmap is drawn.

  For now it is used only for clefs. In future maybe for other objects 
  (in canvas) which have rectangular shape but a custom picture.
 */
class ImageItem: public QCanvasRectangle
{// for drawing clefs
 public:
  //! constructor; the first argument is a QImage to be shown on canvas.
  ImageItem( QImage img, QCanvas *canvas );
 protected:
  //! Overriden showing method.
  /*! Instead of drawing a solid-color rectangle it shows the image. */
  void drawShape( QPainter & );
 private:
  //! the image to be shown 
  QImage image;
  //! pixmap copy of the image
  QPixmap pixmap;
};

ImageItem::ImageItem( QImage img, QCanvas *canvas )
  : QCanvasRectangle( canvas ), image(img)
{
  setSize( image.width(), image.height() );
  pixmap.convertFromImage(image, OrderedAlphaDither);
}


void ImageItem::drawShape( QPainter &p ) // overridden to draw the pixmap!
{
  p.drawPixmap( int(x()), int(y()), pixmap );
}
