/***************************************************************************
 ** $Id: klearnnotes2-defs.h,v 1.5 2004/05/13 02:57:20 wiecko Exp $
                          klearnnotes2
                          klearnnotes2-defs.h  -  description
                             -------------------
    begin                : Wed Oct  8 15:32:54 CEST 2003
    copyright            : (C) 2003 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KLEARNNOTES_DEFS
#define KLEARNNOTES_DEFS
//#define  _KLN_SHOW_SCHEDULE

// ######################## STAFF   SETTINGS ######################## 
//! \name Staff settings:
//@{
//! a name of a note between 'A' and 'C';
/*! in English notation a note between 'A' and 'C' is 'B' ;
  in German notation a note between 'A' and 'C' is 'H',
  and 'B' means 'H flat';

  set defHname to 'H' or 'B'

  but in fact: if it is `H' then the program uses German notation,
  and if it is anything other then `H' the program uses
  English notation
*/
#define defHname 'H'

//!     max no. of NOTES above the five LINES+1 
/*! (that is, above G2 in a treble clef)

note: this will be subtracted from an index to get G2 = note_0

default: 5  ( gives right range for treble clef for guitar)
*/
#define defMAXabove 5   

//!      max no. of all NOTES
/*! default: 23 ( gives right range for treble clef for guitar) */
#define defMAXall 23    

//! number of lines above the five main staff lines
/*!       = int( (defMAXabove+1)/2 )

For the default \def defMAXabove==5 set defNOofLINESabove to 3.

There is a check of consistency of these definitions: if they are inconsistent
the program tries to lower one of the numbers (not to exceed arrays' sizes);
if this cannot be done program exits.
*/
#define defNOofLINESabove 3

//! number of lines below the five main staff lines
/*!       = int( (MAXall-MAXabove-10)/2 )

For \def defMAXabove==5 and defMAXall==23 set defNOofLINESbelow to 4 .

There is a check of consistency of these definitions: if they are inconsistent
the program tries to lower one of the numbers (not to exceed arrays' sizes);
if this cannot be done program exits. 
*/
#define defNOofLINESbelow 4


//! scale for all graphics in canvas;
/*! SC*2 == spacing between 2 lines

default: 10  (good for 1024x768 screen)

Can be overridden by a command line --scale option.
*/
#define defSC 10 

//! a color for temporary nameButton frame highlight
#define defNiceBlue QColor(16,34,188)
//@}

// ###################### BEHAVIOR SETTINGS ###################### 
//! \name Behavior settings
//@{

//! basic time scale; e.g. time [in ms] of a pause between questions
/*! This is used for many timings:
 *    - time to wait before asking another question
 *    - how long a note has a changed  color after nameButton click
 *    - 1/5 of time before statusbar is reset (emptied)

 default: 1000 (==1 second)
*/
#define INTERVAL 1000  	

//! max number of questions ahead, for which a question can be force-scheduled
/*!  When one makes a mistake, the same question is scheduled for
  future frequent use;
  MAXseria keeps max no. of future questions for which 
  an ahead-plan can be scheduled; i.e. if it is impossible to
  schedule a forced question within next MAXseria questions the
  question will not be scheduled

  default: 30
*/
#define defMAXseria 30


//! a base for calculating default test lengths
/*! defTESTlength = length of (number of questions in) a test with
  6 notes active;

  For defTESTlength=60 actual calculated default test lengths changes from 33.333 for 2 notes 
  through 60 for 6 test notes up to 180 for 24 notes active.

  Note, that one can click "Stop"  anytime; otherwise "nextNote" loop will
  end after the calculated  questions.

  default: 60
*/
#define defTESTlength 60

//! easy speed goal for two test notes names active
/*! Min no. of correct answ/(10 min) to receive suggestion 
  "go to next exercise"  and a green frame around "speed";
  below this number one gets "keep practising" and a red frame.

  In future it should be used  for more fancy things (speed of moving 
  notes in the game? limited-time-for-answer exercises?).

  600 answ/10min=60answ/min=1answ/sec which is a good goal
  but if you add time needed to find with your mouse a button this
  is almost impossible even for 2 kinds of notes.
*/
#define defSPEEDgoal02 100
//! normal speed goal for two test notes names active
/*! See comments at \def defSPEEDgoal02. */
#define defSPEEDgoal12 300
//! hard speed goal for two test notes names active
/*! See comments at \def defSPEEDgoal02. */
#define defSPEEDgoal22 500

//! easy speed goal for all seven test notes names active
/*! See comments at \def defSPEEDgoal02. */
#define defSPEEDgoal07  80
//! normal speed goal for all seven test notes names active
/*! See comments at \def defSPEEDgoal02. */
#define defSPEEDgoal17 200
//! hard speed goal for all seven test notes names active
/*! See comments at \def defSPEEDgoal02. */
#define defSPEEDgoal27 300

//! The number of the last of preset lessons.
/*! Some dialogs suggest the user to select the last lesson (because at this lesson all testnotes
  are active - this is a good lesson for playing with sliders/checkboxes to see what's they are about).
  Now, in new versions of kln2 new lessons were added. This definition is used only not to have to 
  change a number of places in the code: whenever we want to refer to the last lesson we use 
  defALLnotexEXEnum, and when extra lessons will be added this definition is the only place 
  which would need changing.
*/
#define defALLnotesEXEnum 44
//@}

// ######################## MUSIC  SETTINGS ######################## 
//! \name Music settings:
//@{

//! initial octave bias (shift of tunes played)
/*!      if you set this to zero you get standard tuning;

but for example guitar notes are usually written in treble clef, while the
guitar is tuned one octave lower; if you learn notes to play guitar
set OCTAVEbias to '-1' - notes in the treble clef will sound like those
played on the guitar

this is just an initial value - user can change it in the sound options dialog

default: -1  (for guitar notes on treble clef - bass clef is too low :( )
default: 0   (for standard tunes)
*/
#define defOCTAVEbias -1
//! actual octave bias (shift of tunes played)
/*! See \def defOCTAVEbias for some comments */
extern int OCTAVEbias;

//! initial value of MIDI patch used to play note's tunes
/*! This is an initial value which can be changed by an user in sound options dialog.

the same patch is used for all notes;

0=piano; 24=nylonGuitar etc. (standard Midi instrument numbers)
*/
#define defMIDIpatch 24
// actual value of MIDI patch used to play note's tunes
/* See \def defMIDIpatch for some comments */
//extern int midiPatch;

//! initial length of note's tune
/* The value in milliseconds [100 == 1 second]. It can be changed by an user in the sound options dialog.

default: 120
*/
#define defNOTElength 60
// actual length of note's tune
//extern int midiNoteLength;
//@}

//! how long widgets stay highlighted (in microseconds)
#define TIMEOFHIGHLIGHT 1000

#endif
