/***************************************************************************
 ** $Id: klearnnotes2.h,v 1.24 2004/05/13 02:57:20 wiecko Exp $
                          klearnnotes2
                          klearnnotes2.h  -  description
                             -------------------
    begin                : Wed Oct  8 15:32:54 CEST 2003
    copyright            : (C) 2003 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/**************************************************************************
 * 
 * a general idea, names of some functions/slots (checkanswer/redraw)
 * and the way notes/lines are numbered comes from 
 * 
 *          Dominik Seichter's       klearnnotes 
 *
 **************************************************************************/

#ifndef KLEARNNOTES_H
#define KLEARNNOTES_H
//#define _KLN_SHOW_SCHEDULE
#define _WITH_VOICE_CONTROL
#include "klearnnotes2-defs.h"
#include <config.h>   
#include <qwidget.h>
#include <qcanvas.h>
#include <qpushbutton.h>
#include <qvariant.h>
#include <qmainwindow.h>
#include <qaction.h>
#include <qmap.h>

#include "midi.h"

class QTimer;
class QLCDNumber;
class QTimer;
class QSlider;
class QComboBox;
class QLabel;
class QCheckBox;
class QProcess;
class QActionGroup;
class QToolBar;
class QPopupMenu;
class QToolButton;
class QActionGroup;
class QStatusBar;
class QSpinBox;
class QFile;
class QGroupBox;
class QTextEdit;

class ClickableView;
class StaffLine;
class Note;

class ActionWithIconSwitch;
class FancyButton;
class FancyQLCDNumber;
class KHelpMenu;

class Exercise;
class ExercisePianoToName; //just defining friends

#include "basemusicnote.h"

///////////// GLOBALS
#include "globals.h"

extern MidiOutput* midiOutput;
///////////////end GLOBALS

//! Map containing the lesson names to be shown in the combo box
/*! The key in this map is the lesson number, the value is a pair containing
  the lesson name and its index in the combo box.
*/
class LessonNameMap: public QObject
{
    Q_OBJECT 

public:
    LessonNameMap();
    QString getLessonName(int lessonNum);
    void setComboIndexForLesson(int lessonNum, int comboIndex);
    void updateComboLabels(QComboBox* combo);
    void populateMapFromNotation(const ScaleNotation& notation);

private:
   typedef QMap<int, QPair<QString,int> > LessonMapType;
   LessonMapType lessonMap;
};

//! a class for the main KLearnNotes2 window

class KLearnNotes2 : public QMainWindow
{
  Q_OBJECT
    public:
  //! a constructor
  KLearnNotes2(QWidget* parent=0,const char *name=0,WFlags fl=WType_TopLevel);
  //! a destructor
  ~KLearnNotes2();


  //  friend class ExercisePianoToName;//::ExercisePianoToName (KLearnNotes2 *);

  //! a method which checks if a test is started
  /*! This method returns true if a test is in progress and false 
    otherwise.

    In future it will have to be redefined to return 0 if there is no
    test started and a `kind of exercise' number if a test is started.
  */
  bool isStarted();
  //! a method which highlights a button for a specified time
  /*! This method highlights (= changes frame color of ) a namebutton
    (= pushButton with note's name). The first argument is a number of the
    button (0 for the leftmost, 1 for the second up to 6 for the last one),
    the second argument dictates how long the button will stay highlighted.
  */
  void pushButtonChangeColorTmp(int buttonNo,int time=INTERVAL);
  //! a method which adds a filename to voice recognition command line arguments
  /*! This method sets up command lines arguments for voice recognition
    QProcess. The only argument is voice model filename. Voice recognition
    is called by<BR>
    <EM>cvoicecontrol newName</EM><BR>
    and its standard out is captured.
  */
  void setVoiceModelFilename(QString newName);// note: this adds this to the
  //           voiceProcess arguments!


  //! midiOutput is a globa variable anyway; is it needed???? [FIXME]
  MidiOutput* getMidiOutput();

 signals:
  //! emitted when a user presses a key which corresponds to a note
  /*! this depends on selected notation (e.g. D in English is Re in Solfeggio;
    D (Do) in Solfeggio is C in English ;

    unrecognized kbd input should be emitted too, to allow additional kbd
    inputs (like number keys for name buttons in v1.1) [FIXME] */
  void keyPressed(BaseMusicNote *);
  //! notes' naming convention changed-"dear widgets, update yourself, plz"
  void notationChanged();

  public slots:
    //! A slot which adds a comment to comments box.
    void comment(QString txt,  QColor commentsBgColor = QColor(255,255,255));
  void resetStatistics( int yourgoal=300);
  void answerWasRight(BaseMusicNote*,int timeSpent,bool wasItRight);
  //!disables some menus
  void prepareForTestStart();
  //!enables what prepareForTestStart() disabled
  void prepareForTestStop();

  protected slots:
    //! a slot which draws next question
    /*! This slot is used to draw (=randomly choose) next question in a test.
     */
    void nextNote();
  // MENU/TOOLBAR SLOTS:
  //! \name Menus and Toolbars slots:
  //@{
  //! a slot which shows 'test length factor' dialog
  void dialogTestLengthFactor();
  //! a slot which shows MIDI (sound) setup dialog
  void dialogSetupMidi();
#ifdef _WITH_VOICE_CONTROL
  //! a slot which shows 'microphone calibration' dialog
  /*! This is used for voice recognition. Before an user can use voice 
    recognition he has to calibrate his microphone. 
  */
  void dialogVoiceMircoSetup();
  //! a slot which shows 'voice model editor' dialog (for voice recognition)
  /*! In this dialog user can 
   *    - choose one of existing voice model files or start a new model
   *    - record or delete samples for voice commands
   *    - save modified voice model.
   */
  void dialogVoiceModelEditor();
#endif
  //! a slot to change the active notation
  /*! This method changes the currently selected notation. It is connected to 
    a menu action toggle() signal.
  */
  void setNotationByAction(QAction*);
  //! a slot showing 'About' dialog
  /*! This method shows html version of the dialog and should be used 
    only if KDE help doesn't work on this system. */
  void mH_doesntWork_about_activated();
  //! a slot showing Help index
  /*! This method shows html version of the dialog and should be used 
    only if KDE help doesn't work on this system. */
  void mH_doesntWork_index_activated();
  //! a slot showing Quick Start helpfiles
  /*! This method shows html version of the dialog and should be used 
    only if KDE help doesn't work on this system. */
  void mH_doesntWork_quickStart_activated();
  //! a slot showing Voice Recognition Quick Start helpfiles
  /*! This method shows html version of the dialog and should be used 
    only if KDE help doesn't work on this system. */
  void mH_doesntWork_voiceQuickStart_activated();
  //! a slot showing Help index
  /*! This method shows the default, KDE help version of helpfiles. */
  void mH_KHelp_activated();
  //! a slot showing Quick Start helpfiles
  /*! This method shows the default, KDE help version of helpfiles. */
  void mH_quickStart_activated();
  //! a slot showing Voice Recognition Quick Start helpfiles
  /*! This method shows the default, KDE help version of helpfiles. */
  void mH_voiceQuickStart_activated();
  //! a slot showing 'About' dialog
  /*! This method shows the default, KDE help version of helpfiles. */
  void mH_about_activated();
  //! a slot used to turn MIDI sound on or off
  /*! If run with onOrOff==true it tries to turn MIDI sound on. 
    If onOrOff==false it turns the sound off. */
  void turnSoundOnOff(bool onOrOff);
#ifdef _WITH_VOICE_CONTROL
  //! a slot used to turn MIDI sound on or off by a voice command
  /*! It is activated when voice command is recognized.
    If run with onOrOff==true it tries to turn MIDI sound on. 
    If onOrOff==false it turns the sound off. */
  void voiceTurnSoundOnOff(bool onoroff);
#endif
  //! a slot to change clef to treble (true) or bass (false)
  /*! This method changes clef to either treble (when the argument
    treble==true) or bass (when treble==false). It is connected to 
    a menu action.
  */
  void setClefByAction(bool treble); // by action
  //! a slot to change clef to treble (0), bass (1), alto(2), tenor(3)
  /*! This method changes clef to either treble (when the argument
    indeks==0) or bass (when indeks==1). It is connected to 
    a combo box.
  */
  void setClefByCombo(int indeks);
  //! a slot to change speed goal
  /*! This method changes speed goal ("level") to easy ("Prentice"),
    normal ("Journeyman") or hard ("Wizard"). It is connected to 
    a menu action toggle() signal.
  */
  void setLevelByAction(QAction*);
  //! a slot to change speed goal
  /*! This method changes speed goal ("level") to easy ("Prentice"),
    normal ("Journeyman") or hard ("Wizard"). It is connected to 
    a combo box.
  */
  void setLevelByCombo(int);
  //! a slot to setup custom speed goal
  /*! This method opens a dialog, which enables setting custom
    speed goals. It is connected to a menu action activated() signal,
    because it should be run even if the action was not toggled, that
    is even if a user practised at a custom level, clicking on the
    menu item should open a custom speed goal dialog.
  */
  void a_mo_setLevelCustom_activated();
  //! user chose different kind of exercise
  void chooseExercise(QAction* a);
  //! a slot saving user settings
  void saveOptions();
  //! a slot loading user settings
  void loadOptions();
  //! a slot used before the destructor is called
  /*! The destructor is to do only basic cleanup. There are some things
    it cannot run/activate. Therefore a regular slot endKLearnNotes2()
    is run just before the destructor.
  */
  void endKLearnNotes2();

  //! a slot which sets up preset lessons(sets of active testnotes)
  /*! This slot is run whenever user chooses another lesson from the
    lessons combo box. It sets name checkboxes and range sliders to
    one of the preset values and runs setupActiveNotes()
  */
  void activeNotesChangedByLesson(int indeks);
  //! a slot which sets active testnotes when one uses sliders or checks
  /*! This slot is run whenever user changes active testnotes by
    sliding a slider or checking/unchecking a checkbox with her mouse.
    It simply runs setupActiveNotes(), ignoring the 'notimportant' argument.

    It is defined only because setupActiveNotes() has wrong arguments,
    and therefore cannot be connected to some signals.
  */
  void activeNotesChangedManual(int notimportant); 

  //! a slot for starting and stopping a test
  /*! An user can start and stop by clicking on the same button. This button's
    'clicked()' signal is connected to slot startStop().

    If there is a test running, this slot stops it. If there is no test, this
    slot starts a new one.
  */
  void startStop();
  //! menu action should run a different exercise depending on which one is on
  void aStartStopActivated();
  //! a slot which chooses next lesson (=preset set of testnotes)
  /*! This slot chooses next lesson (=preset set of testnotes). It is connected
    to an action aNextLesson (which is both a button in a toolbar and an item
    in a menu).
  */
  void nextLesson();

  //  void setTestLengthFactor(int number); // old one: it was used long time
  // ago when user could change test length by a spinbox in a toolbar
  //@}

  // ANSWER SLOTS: 
  //! \name Answer slots:
  //@{
  //! an overridden slot connected to key press event
  /*! This slot is run whenever user presses a key. If the key is one
   of acceptable ones, checkanswer(keyname) is run.
  */
  void keyPressEvent( QKeyEvent *k );
  //! a slot activated when user clicks on pushButton 'C'
  /*! It just runs checkanswer('C'). The checkanswer ('C') couldn't be
    connected to pushButton clicked() signal, because they have different
    argument type.
  */
  void noteCclicked();
  //! a slot activated when user clicks on pushButton 'D'
  /*! See noteCclicked() for extra comments. */
  void noteDclicked();
  //! a slot activated when user clicks on pushButton 'E'
  /*! See noteCclicked() for extra comments. */
  void noteEclicked();
  //! a slot activated when user clicks on pushButton 'F'
  /*! See noteCclicked() for extra comments. */
  void noteFclicked();
  //! a slot activated when user clicks on pushButton 'G'
  /*! See noteCclicked() for extra comments. */
  void noteGclicked();
  //! a slot activated when user clicks on pushButton 'A'
  /*! See noteCclicked() for extra comments. */
  void noteAclicked();
  //! a slot activated when user clicks on pushButton 'H' or 'B'
  /*! See noteCclicked() for extra comments.

  Note, that the same slot is used for note 'H' in German notation
  and 'B' in English notation (which is the same note).*/
  void noteHclicked(); 
  //@}

  // TIMER SLOTS:
  //! \name Timer slots:
  //@{
  //! a slot which grows 'time spent' counter during a test
  /*! This slot is run whenever the main test clock reaches a certain value.
    For now, it only adds one to time counters, and displays them in the 
    results box (lower-right in the main window). In future for some kinds
    of exercises it may e.g. move Note's position (something like running
    currentNote->advance()?) */
  void growtime();
  //  void resetComments(); // old slot
  //! a slot used for a pause before next exercise is asked
  /*! In the very end of checkanswer() it displays whether user answered right
    or wrong and a singleShot QTimer is created and program returns to the main
    loop. After some time (default=INTERVAL) the timer runs redraw() slot, 
    which does come clean up and runs nextNote().

    The result is that while users results (right/wrong) are displayed user
    can interact with program, e.g. click 'Stop' to stop test. Using e.g.
    sleep() to wait specific time would cause program to freeze, which 
    we wanted to avoid.

    You may think of redraw() as about the final part of checkanswer().
  */
  void redraw();
  //@}

#ifdef _WITH_VOICE_CONTROL
  // voice recognition slots:
  //! \name Voice recognition slots:
  //@{
  //! a slot which reads input from voice recognition standard out
  /*! If user said note name checkanswer() is run.

  If user said a commands name (like 'Sound off'), appropriate methods
  are called.
  */
  void readFromVoice();
  //! a slot which starts/stop voice recognition
  void voiceStartStop(bool);
  //! a slot activated when voice recognition process ends (exits)
  void voiceExited();
#endif

 protected:
  //! a method which setups Lessons combo box
  /*! For now it is called only once, by the KLearnNotes2() creator,
    but potentially useful to setup lessons for different kinds of 
    exercises (e.g. fretboard etc.). */
  void setupLessons();

  //! a map holding all lesson names, and where in the combo box that name is stored
  LessonNameMap allLessons;
  
  //! a method checking if a pushButton with this name (e.g. 'C') is active
  /*! It returns true if this namebutton is active (i.e. a chceckbox under it
    is checked) or false it the namebutton is not active.
    
    It is used in keyPressEvent() to check if should we consider a keyboard
    answer and send it to checkanswer().
  */
  bool noteNameIsActive(char);

  //! a method setting up notes for test based on sliders/checkboxes
  /*! This method sets checks which of the notes fulfil range sliders and
    name checkboxes requirement.

    These notes are put into activeNotes[] array and their number is 
    stored in numActiveNotes. Also, all active notes are made visible.
  */
  void setupActiveNotes();

  //! a method for reacting on user input
  /*! It is run by keyPressEvent(), readFromVoice(), and noteCclicked().
    The argument is name of a note (e.g. 'C').

  If a test is running the input is considered an answer and it is checked
  if the answer is correct.

  If no test is running the input causes highlighting of all notes of this name
  and their name push button.
  */
  void checkanswer(char);

#ifdef _KLN_SHOW_SCHEDULE
  //! for debugging and info: a method showing forced scheduled questions
  /*! This method displays on stdout contents of suggestion[] array.
    See 'devel: How does it work?' section of kln2 helpfiles.
  */
  void debugShowSuggestions(int activequenum=-1);
#endif


  // ACTIONS, MENUS
  //! \name Actions, menus
  //@{
  //! a method which shows custom speed goal setup dialog
  /*! A method which shows custom speed goal setup dialog. It is run by
    slots connected to custom speed goal action in menu options and to
    custom speed item in level combobox.
  */
  bool dialogCustom_show();
  //! A method which sets up speed goal for a specific lesson.
  /*! A method which sets up speed goal based on level and number of 
    active namebuttons.
  */
  void setupSpeedGoal();
  //! A method which returns speedgoal for 2 namebuttons active.
  /*! A method which returns speedgoal for a specific level when there are 
     2 namebuttons active.
  */
  int speed2(int level);
  //! A method which returns speedgoal for 7 namebuttons active.
  /*! A method which returns speedgoal for a specific level when there are 
     7 namebuttons active.
  */
  int speed7(int level);
  //! A method which sets custom speed goal without showing a dialog.
  /*! This method is run in the following situation:
   *   - user used custom speed goal
   *   - then she clicked on 'custom level' to change her custom speed goal
   *   - then she clicked cancel.

   In such a situation program falls back to previous settings i.e. the
   old custom speed goal. But the custom dialog should not be shown
   again.
  */
  void setupLevelCustom_withoutDialog();
  
  //! Last speed goal: easy (Prentice), medium (Jman), hard (Wizard) or custom.
  /*! Used for fall-back when user click cancel in custom speed goal setup
    dialog.
  */
  int lastLevel; // if user has chosen "custom level" and then clicked 
  //                "cancel" we should go back to the previous level;
  //                 lastLevel remembers it's number


  //! A method which sets up test length for a specific lesson.
  /*! This method sets test length for a specific test. The length depends
     on number of active notes, and may be increased or diminished if
     user's testLengthFactor is not equal to 100.
  */
  void calculateTestLength();

  //! true if we are loading setting from a file
  /*! For now it is only used to prevent showing custom level dialog if
     custom speed goal is set up according to the saved configuration.
   */
  bool restoring;

  QMenuBar     *menubar;
  QPopupMenu   *menuPractice;
  QPopupMenu   *menuOptions;
  QPopupMenu   *menuHelp;
  KHelpMenu    *menuHelpK;

  QPopupMenu    *mp_Cleves_menu;
  QActionGroup  *mp_Cleves_grp;
  ActionWithIconSwitch    *a_mp_SetClefTreble;
  ActionWithIconSwitch    *a_mp_SetClefBass;
  QPopupMenu    *mp_Exercises_menu;
  QActionGroup  *mp_Exercises_agroup;
  ActionWithIconSwitch    *a_mp_exercise[4];
  //! array of all availaboe exercises
  /*!  for now: 0 == the old one (note in the staff to name),
	1 == game based on old exercise;
    2 == new: piano kbd key to name
    3 == new: name to piano kbd key */
  Exercise* exercises[4];
  Exercise* currentExercise;
  QAction              *a_mp_Quit;

  QPopupMenu   *mo_Levels_menu;
  QActionGroup  *mo_Levels_grp;
  ActionWithIconSwitch    *a_mo_setLevel[4];

  ActionWithIconSwitch *a_mp_SetNotationEnglish;
  ActionWithIconSwitch *a_mp_SetNotationGerman;
  ActionWithIconSwitch *a_mp_SetNotationSolfeggio;
  
  QAction     *a_mo_setTestLength;
  ActionWithIconSwitch    *a_mo_soundOnOff;
  QAction    *a_mo_soundSetup;

#ifdef _WITH_VOICE_CONTROL
  QPopupMenu   *mo_Voice_menu;
  QAction    *a_mo_voiceMicroSetup;
  ActionWithIconSwitch    *a_mo_voiceOnOff;
  //! Used to fork a voice recognition process.
  /*! voiceProcess keeps data about voice recognition program 
    (cvoicecontrolkln2). It is used to fork voice recognition when it should
    be started and to make it exit when user wants to stop it. We
    capture forked process's standard out (see readFromVoice()).
  */
  QProcess   *voiceProcess;
  //! A file holding current voice model (speaker model with recorded samples).
  QFile      *voiceModelFile;
  QAction    *a_mo_startVoiceEditor;
#endif

  QAction    *a_mo_save;
  QAction    *a_mo_load;

  QAction    *a_mH_KHelp;
  int id_mH_help;
  QAction    *a_mH_KQuickStart;
  QAction    *a_mH_KVoiceQuickStart;
  //! Holds help menu items referring to HTML backup copy of helpfiles.
  /*! I was not able to prepare install in such a way that on all systems
        KDE help-file was found by KDE help browser. Therefore, I included
        in this distribution a HTML copy of index.docbook. You can access
        HTML version of KLearnNotes2 help-files through items in this
        submenu.
  */
  QPopupMenu *mH_doesntwork_menu;
  //! Hold id of mH_doesntwork_menu submenu.
  /* Used to reuse mH_doesntwork_menu text (and translations). */
  QString    mH_doesntwork_path;
  int id_mH_doesnt;
  QAction    *a_mH_about_doesntwork;
  QAction    *a_mH_index;
  QAction    *a_mH_quickStart;
  QAction    *a_mH_voiceQuickStart;
  QAction    *a_mH_about;


  QStatusBar  *statbar;


  QToolBar    *programToolBar;
  QToolBar    *exerciseToolBar;
  QToolButton *tbWhat;
  QPushButton* bStartStop ;
  QAction*     aStartStop ;
  QAction*     aNextLesson;
  QComboBox*   combo_lesson ;
  //! Used to "slide off" the unused separators of lessons combo.
  int lastLesson;
  QComboBox*   combo_clef ;
  QComboBox*   combo_level ;
  //  QSpinBox*    SB_testLength;
  //@}

  // buttons, canvas, sliders, labels:
  //! \name Main window widgets (buttons/canvas...)
  //@{
  //! the main window main layout (horizontal)
  QBoxLayout * layoutMain;
  //! the main left layout (vertical); top = quiestion; buttom = answer
  QBoxLayout * layoutMainLeft;
  //! temporary: a qframe holding v.1.1 note to name exercise
  QFrame* frameWithOldExercise;
  //! central widget of the main window
  QWidget * klnCentralWidget; 

  QCanvas* c;
  //! QCanvasView with overridden mouseClickEvent().
  ClickableView* drawArea;
  //! Ranges slider1: sets range of active notes (testnotes).
  QSlider* minmax1;
  //! Ranges slider2: sets range of active notes (testnotes).
  QSlider* minmax2;

  //! are changes in sliders/checkboxes done by a preset lesson?
  /*! Whenever sliders are slided by an user, lesson combo is reset to
    "custom lesson". Now, if user selects one of preset lessons sliders are
    moved too, but this should not change anything in lessons combo.

    This variable is true if rangesliders are adjusted by one of preset
    lessons, and false if user change their state manually (mousually?)
  */
  bool autochange;

  //! Labels above namebuttons, showing alternative key input.
  QLabel* nameKeys[7];
  //! Main name pushButtons.
  FancyButton* namePBs[7];
  /*! Name checkboxes; they activate/deactivate name pushButtons.
   */
  QCheckBox*   nameCHKs[7];
  //  QLabel*      labelComments ;
  QGroupBox*   commentsGroup;
  QTextEdit*   commentsText;
  //@}

  // total statistics:
  //! \name Test statistics:
  //@{
  //! current question number (in a test)
  QLabel*      labelQueNum ;
  //! Current question number.
  int quenum;
  //! time spent on current question
  QLCDNumber*  lcd_queTime;
  //! time spent on current question
  /* One could always refer to lcd_queTime value, but putting this in a 
     separate variable makes code a bit easier to read.
  */
  int onetime;

  //! number of correct answers
  QLCDNumber* lcd_numOK;
  //! number of wrong answers
  QLCDNumber* lcd_numWR;
  //! a color used to switch back background color of lcd_numWR
  QColor bgcolor; 
  //! user's speed in current test
  FancyQLCDNumber* lcd_speed;
  //! user's speed goal
  QLCDNumber* lcd_yourGoal;
  //! time spent on all questions in current test, that is time since start()
  int timespent;
  //! main test timer
  /*! Its "ticks" (timeout() signal) make lcd_queTime and timespent grow.
    In future it should also advance note's position (e.g. in the 
    notes-splashing-on-the-clef game).
  */
  QTimer* clocky;

  //! Current question.
  Note* note;

  //! number of active notes which will be used for a test (testnotes)
  int numActiveNotes;
  //! an array of active notes which will be used for a test (testnotes)
  Note* activeNotes[defMAXall];
  //! an array of force-scheduled questions
  Note* suggestion[defMAXseria];
  //! calculated based on testLengthFactor and number of active notes
  int testLength;
  int testLengthFactor;

  //! custom speed goal for 2 nameButtons active
  int customSpeedGoal2;
  //! custom speed goal for 7 nameButtons active
  int customSpeedGoal7;
  //@}

  //! \name Canvas items:
  //@{
  //! Leger lines above the main 5 lines of the staff.
  StaffLine* linesabove [defNOofLINESabove];
  //! Leger lines below the main 5 lines of the staff.
  StaffLine* linesbelow [defNOofLINESbelow];
  //! Short leger lines for easier range sliders usage; hidden during test.
  StaffLine* guidelines [defNOofLINESabove+defNOofLINESbelow];

  //! Clef pictures.
  QCanvasPolygonalItem* clefItem[3];

  Note* allNotes [defMAXall];
  //! Used just as a note previous to the toppest one
  Note* nullNote;

  //! corrected (lowered if inconsistent) value of maxabove
  int numNotesABOVE;
  //! corrected (lowered if inconsistent) value of maxall
   int numNotesALL;
  //! corrected (lowered if inconsistent) value of nooflinesabove
  int numLinesABOVE;
  //! corrected (lowered if inconsistent) value of nooflinesbelow
  int numLinesBELOW;
  //! canvas  y-position of the toppest of the 5 main lines of the staff
  int posOfTopLine;
  //@}
};

#endif
