/***************************************************************************
 ** $Id: main.cpp,v 1.11 2004/05/13 02:57:20 wiecko Exp $
                          klearnnotes2
                          main.cpp -  description
                             -------------------
    begin                : Wed Oct  8 15:32:54 CEST 2003
    copyright            : (C) 2003 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kapp.h>
#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <qtranslator.h>
#include <qtextcodec.h>
#include <qapplication.h>
#include <qfile.h>
#include <qdir.h>
#include <qstringlist.h>

#include "klearnnotes2.h"  // this includes kln-defs.h (SC!)

bool quiet=false;

static const char *shortdescription =
QT_TR_NOOP("KLearnNotes2: Learn music notes' names fast!");
static const char *description =
QT_TR_NOOP("Includes:\n"
	   "* wise learning algorithm, \n"
	   "* treble and bass clefs,\n"
	   "* convenient notes-to-learn selection,\n"
	   "* voice recognition (answer with your mic!)");

KAboutData aboutData( "klearnnotes2", "KLearnNotes2",
		      VERSION, shortdescription, KAboutData::License_GPL,
		      "(c) 2003, Marek Wieckowski", description, 0, 
		      "wiecko AT users.sourceforge.net");	

int SC;

//! command line options
static KCmdLineOptions options[] = 
  {
    { "scale <scale>", QT_TR_NOOP("a scale for graphical objects; default: 10 "
				  ),"defSC"},
    { "q",0,0},
    { "quiet", QT_TR_NOOP("turns off most of program output to stdout and stderr"
			  ),0},   
    { 0, 0, 0 } // End of options.
  };


int main(int argc, char *argv[])
{
  fprintf(stderr,"%s%s%s",
	  "\n KLearnNotes2 ",VERSION," Copyright (C) 2003-2004 Marek Wieckowski\n\n"
	  "KLearnNotes2 comes with ABSOLUTELY NO WARRANTY; \n"
	  "either expressed or implied, including, but not limited to, \n"
	  "the implied warranties of merchantability and fitness for a \n"
	  "particular purpose.\n\n"
	  "For details view file COPYING.\n"
	  "This is free software. You are welcome to redistribute it \n"
	  "under terms of GNU Public Licence 2.0 or later (you choose).\n\n"
	  );

  aboutData.addAuthor("Marek Wieckowski",0, 
		      "wiecko AT users.sourceforge.net");
  aboutData.addAuthor("Dominik Seichter",
		      QT_TR_NOOP("1st author of an underlying klearnnotes"
				 "0.1 project"),"domseichter AT web.de");
  aboutData.addAuthor("Daniel Kiecza",
		      QT_TR_NOOP("the author of voice recognition engine"
				 ),"daniel AT kiecza.de");

  /////////////////
  // new KApplication; start of COMMAND LINE args processing
  /////////////////
  KCmdLineArgs::init( argc, argv, &aboutData );
  KCmdLineArgs::addCmdLineOptions( options );
  KApplication a;
  KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

  // extracting a command line argument as a QString
  QString QSargSC = args->getOption("scale");
  bool ok;
  SC=QSargSC.toInt(&ok,10);
  // if command line is nonsense we use default value:
  if ((SC<1)||(!ok)) SC=defSC;

  if(args->isSet("q"))
    quiet=true;

  args->clear();
  /////////////////
  // end of command line processing
  /////////////////


  /////////////////
  // start of TRANSLATIONS setup
  /////////////////
  static QTranslator *translator = 0;
  translator = new QTranslator( 0 );
  QStringList localesToCheck;
  localesToCheck.push_back(QTextCodec::locale());
  // for LANG=ru_RU:
  // DEBUG: next localesToCheck[]: ru_RU
  // DEBUG: next localesToCheck[]: ru
  // DEBUG: next localesToCheck[]: ru_*
  // - we first look for exact LANG match, then for general this-language file, 
  // then for any translation for this language.
  //
  // First match of the following is loaded:
  // 
  // DEBUG: Searched: HERE/klearnnotes2_ru_RU.qm
  // DEBUG: Searched: HERE/../po/klearnnotes2_ru_RU.qm
  // DEBUG: Searched: SHARE/klearnnotes2_ru_RU.qm
  //
  // DEBUG: Searched: HERE/klearnnotes2_ru.qm
  // DEBUG: Searched: HERE/../po/klearnnotes2_ru.qm
  // DEBUG: Searched: SHARE/klearnnotes2_ru.qm
  //
  // DEBUG: Searched: HERE/klearnnotes2_ru_*.qm
  // DEBUG: Searched: HERE/../po/klearnnotes2_ru_*.qm
  // DEBUG: Searched: SHARE/klearnnotes2_ru_*.qm
  //
  // Because we first look for local files(./kln.qm and ../po/kln.qm) it is
  // easy to test new translation files (without changing global /usr/share/...
  // files).
  //
  if(!(*localesToCheck.begin()).isEmpty())
    {
      int position=(*localesToCheck.begin()).find('_');
      if (position != -1) // '_' was found : we have "es_MX" kind locale
	{
	  QString tmpString(*localesToCheck.begin()) ;
	  tmpString.truncate(position);
	  localesToCheck.push_back(tmpString);
	}
      // now the last in localesToCheck is without a dash; it is like "es"
      QStringList::iterator langName=localesToCheck.end();
      langName--; // now it is the last element!
      localesToCheck.push_back(*langName+"_*");
      QStringList dirsToCheck;
      dirsToCheck.append("./");
      if(QDir("../po/").exists())
	dirsToCheck.append("../po/");
      if(QDir("/usr/share/apps/klearnnotes2/locale/").exists())
	dirsToCheck.append("/usr/share/apps/klearnnotes2/locale/");

      QStringList listOfFiles;
      QString dirName;
      QStringList::iterator i;
      QStringList::iterator j;
      for (i = localesToCheck.begin(); i!=localesToCheck.end() ; i++)
	{
	  for( j=dirsToCheck.begin(); j!=dirsToCheck.end(); j++)
	    {
	      listOfFiles=QDir(*j).entryList(QString("klearnnotes2_") +
					     *i+ ".qm");
	      if (listOfFiles.size()>0)
		{
		  dirName=*j;
		  break;
		}
	    }
	}
      if (listOfFiles.size()>0)
	{
	  debuginfo("Loading translations");
	  translator->load(*listOfFiles.begin(),dirName);
	  a.installTranslator(translator );
	}
    }
  /////////////////
  // end of TRANSLATIONS setup
  /////////////////

  KLearnNotes2 *klearnnotes2 = new KLearnNotes2();
  a.setMainWidget(klearnnotes2);
  // the main window is shown:
  klearnnotes2->show();  
  
  return a.exec();
}
