/***************************************************************************
 ** $Id: midi-OSS.cpp,v 1.3 2004/05/28 15:22:55 wiecko Exp $
                          klearnnotes2
                          midi-OSS.cpp -  description
                             -------------------
    begin                : Sun Feb 29 23:58:22 CET 2004
    copyright            : (C) 2004 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qprocess.h>
#include <qstring.h>
#include <stdlib.h>   //-
#include <stdio.h>    //-
#include <unistd.h>   //-
#include <fcntl.h>    //-
#include <stdarg.h>
#include <sys/types.h>

#include <sys/ioctl.h>      // for writing to the device 
#include <sys/soundcard.h>  // the soundcard macro definitions 


#include "klearnnotes2-defs.h"
#include "globals.h"
#include "midi-OSS.h" // this includes midi!


// standard code from incude/linux/soundcard.h :
int midifd = -1;
SEQ_DEFINEBUF(1024);
//! standard OSS/Free code from incude/linux/soundcard.h
void seqbuf_dump()
{
  if (_seqbufptr)
    write (midifd, _seqbuf, _seqbufptr);
  _seqbufptr = 0;
}


//! cretator; sets up default values
OSSMidiOutput::OSSMidiOutput() : MidiOutput()
{
  //  debuginfo("OSSMidiOutput creator: start");
  internal_midifd = -1;
  mixerfd=-1;
  
  midiOutputName=QString("OSS/Free");
  //  debuginfo("OSSMidiOutput creator: end");
  noOfNotesToPlay=0;
}

OSSMidiOutput::~OSSMidiOutput(){}

void OSSMidiOutput::addNoteToChord(int pitch)
{
  //  debuginfo("OSS: add a note");
  if(::midifd==-1)//device is NOT opened
    return;
  notesToPlay[noOfNotesToPlay]=pitch;
  noOfNotesToPlay=(noOfNotesToPlay+1)%maxNotesInAChord;
}

void OSSMidiOutput::playChord()
{
  //  debuginfo("OSS: play");
  if((noOfNotesToPlay>0)&&isOn())
    {
      int mypid; // FORK FORK FORK 
      if ((mypid=fork())==0)
	{  
	  //	  debuginfo("Hi, I'm FORKED...");
	  int newVel=60;
	  int midiCurrentChannel=0;
	  for(int i=0;i<noOfNotesToPlay;i++)
	    {
	      SEQ_START_NOTE(midiCurrentDevice, midiCurrentChannel,
			     notesToPlay[i], newVel);
	      midiCurrentChannel=(midiCurrentChannel+1)%midiMaxChan;
	      // notes are added to different channels
	    }
	  SEQ_DELTA_TIME(noteLength);
	  seqbuf_dump();        
	  ioctl(midifd, SNDCTL_SEQ_SYNC);
	  midiCurrentChannel=0;
	  for(int i=0;i<noOfNotesToPlay;i++)
	    {
	      SEQ_STOP_NOTE(midiCurrentDevice, midiCurrentChannel,
			    notesToPlay[i], 64);
	      // Uses 0x80, true MIDI note-off 
	      midiCurrentChannel=(midiCurrentChannel+1)%midiMaxChan;
	    }
	  seqbuf_dump();
	  ioctl(midifd, SNDCTL_SEQ_SYNC);
	  //	  debugtxt("... FORKED ending.");
	  _exit(0);  // EXIT CHILD PROCESS AFTER PLAYING A CHORD
	}
    }
  noOfNotesToPlay=0;
}



void OSSMidiOutput::setupPatches()
{
  for(int i =0; i< midiMaxChan; i++)
    SEQ_SET_PATCH(midiCurrentDevice,i,midiPatch);
}

bool OSSMidiOutput::turnOn()
{
  //  debuginfo("OSS: turn ON");
  if (isOn()) 
    {
      //      debuginfo("midi_turnOn(): but midi IS already on!");
      return true;
    }
  suspendArts();
  //  debuginfo("MIDI: OPENING DEVICE!");
  if ((midifd = open(MIDI_DEV, O_WRONLY)) == -1)
    {
      //      debuginfo("midi_turnOn(): couldn't open midi device!");
      internal_midifd=-1;
      return false;     //  Can't open MIDI device
    }
  internal_midifd=midifd;
  //  debuginfo("MIDI: device is open! Starting timer.");
  SEQ_START_TIMER();
  //  debuginfo("MIDI: Setting tempo.");
  SEQ_SET_TEMPO(100);
  //  debuginfo("MIDI: Setting up patches.");
  setupPatches();
  //  debuginfo("MIDI: opening mixer.");
  if( (mixerfd = open("/dev/mixer", O_WRONLY, 0))==-1 )
    {
      //      debuginfo("midi_turnOn(): couldn't open mixer device!");
      return true; // can't open mixer but /dev/sequencer is open!
    }
  //  debuginfo("MIDI: setting up mixer.");
  mixerSetMasterVolume(midiCurrentMasterVolume);
  mixerSetMidiVolume(midiCurrentVolume);
  //  debuginfo("midi_turnOn(): midi and mixer open");
  return true;
}


void OSSMidiOutput::turnOff()
{
  //  debuginfo("OSS: turn OFF");
  if (midifd != -1)
    {
      close(midifd);
      midifd=-1;
    }
  internal_midifd=-1;
  if (mixerfd != -1)
    {
      close(mixerfd);
      mixerfd=-1;
    }
}



bool OSSMidiOutput::mixerIsOpen()
{
  if(mixerfd==-1)
    {
      return false;
    }
  return true;
}

void OSSMidiOutput::mixerSetMasterVolume(int newvol)
{
  midiCurrentMasterVolume=newvol;
  if(midiCurrentMasterVolume>100) midiCurrentMasterVolume=100;
  if(midiCurrentMasterVolume<0) midiCurrentMasterVolume=0;
  if(mixerfd!=-1)
    {
      int level = (midiCurrentMasterVolume << 8) + midiCurrentMasterVolume;
      ioctl( mixerfd, MIXER_WRITE(SOUND_MIXER_VOLUME),   &level );
      midiCurrentMasterVolume=level & 0xff;
      // above = returned value for left voulume;
      // right = (level & 0xff00) >> 8;
    }
}

void OSSMidiOutput::mixerSetMidiVolume(int newvol)
{
  midiCurrentVolume=newvol;
  if (midiCurrentVolume>100) midiCurrentVolume=100;
  if (midiCurrentVolume<0) midiCurrentVolume=0;
  if(mixerfd!=-1)
    {
      int level = (midiCurrentVolume << 8) + midiCurrentVolume;
      ioctl( mixerfd, MIXER_WRITE(SOUND_MIXER_SYNTH),   &level );
      midiCurrentMasterVolume=level & 0xff;
      // above = returned value for left voulume;
      // right = (level & 0xff00) >> 8;
    }
}

int  OSSMidiOutput::mixerMasterVolume()
{
  if(mixerIsOpen())
    {
      int level;
      ioctl( mixerfd, MIXER_READ(SOUND_MIXER_VOLUME), &level );
      if((level& 0xff) != midiCurrentMasterVolume)
	{
	  //	  printerror("Master volume changed by some other program!\n using new settings");
	  midiCurrentMasterVolume=level& 0xff;
	}
    }
  return midiCurrentMasterVolume;
}

int  OSSMidiOutput::mixerMidiVolume()
{
  if(mixerIsOpen())
    {
      int level;
      ioctl( mixerfd, MIXER_READ(SOUND_MIXER_SYNTH), &level );
      if((level& 0xff) != midiCurrentMasterVolume)
	{
	  //	  printerror("Midi volume changed by some other program!\n using new settings");
	  midiCurrentMasterVolume=level& 0xff;
	}
    }
  return midiCurrentVolume;
}
