/***************************************************************************
 ** $Id: midi-TSE.cpp,v 1.6 2004/05/24 21:29:37 silhusk Exp $
                          klearnnotes2
                          midi-OSS.cpp -  description
                             -------------------
    begin                : Sun Feb 29 23:58:22 CET 2004
    copyright            : (C) 2004 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <config.h>
#ifdef WITH_TSE3 // HAVE_TSE3_TSE3_H


// #include <qprocess.h>
// #include <qstring.h>
// #include <stdlib.h>   //-
// #include <stdio.h>    //-
#include <unistd.h>   // fork
// #include <fcntl.h>    //-
// #include <stdarg.h>
// #include <sys/types.h>

// #include <sys/ioctl.h>      // for writing to the device 
// #include <sys/soundcard.h>  // the soundcard macro definitions 


//#include "klearnnotes2-defs.h"
#include "globals.h"
#include "midi-TSE.h" // this includes midi!


//! cretator; sets up default values
TSEOSSMidiOutput::TSEOSSMidiOutput
(TSE3::Plt::UnixMidiSchedulerFactory::UnixPlatform platform) :
  MidiOutput(), port(TSE3::MidiCommand::NoPort)
  //  transport(&metronome, &scheduler)
{
  internal_midifd=-1;

  time=0 ;
  setMidiMaxChan(16);
  debuginfo("TSEOSSMidiOutput creator: start");

  TSE3::MidiSchedulerFactory msf(true);
  // chose the platform
  TSE3::Plt::UnixMidiSchedulerFactory::setPreferredPlatform (platform);
  // and create the scheduler using the factory
  scheduler = msf.createScheduler();

  //debuginfo(QString("  created %1").arg(scheduler->implementationName()));
  debuginfo(QString("  preferred is %1").arg(TSE3::Plt::UnixMidiSchedulerFactory::preferredPlatform()));
  debuginfo(QString("  created %1").arg(TSE3::Plt::UnixMidiSchedulerFactory::createdPlatform()));

  transport = new TSE3::Transport(&metronome, scheduler);
  setMidiCurrentDevice(getMidiCurrentDevice()); // set 'port'

  mixer = new TSE3::Mixer(scheduler->numPorts(), transport);

  // initialize volume value
  //(*mixer)[midiCurrentDevice]->setVolume(midiCurrentVolume);

  midiOutputName=QString("OSS/TSE");
  debuginfo("TSEOSSMidiOutput creator: end");
}

TSEOSSMidiOutput::~TSEOSSMidiOutput()
{
  turnOff();
  delete mixer;
  delete transport;
  delete scheduler;
}

void TSEOSSMidiOutput::addNoteToChord(int pitch)
{
  //  debuginfo(QString("TSEOSS: add a note %1").arg(pitch));
  time=0 ; // always start at the beginning of the chord
  //  note=rootNote;
  //  int delta[2]  = {pitch, 0};
  //const int velocity = 0x60;
  const int channel  = 0;//TSE3::MidiCommand::AllChannels;
  //const int port     = -2;

  note=pitch;

  phraseEdit.insert
    (TSE3::MidiEvent(TSE3::MidiCommand(TSE3::MidiCommand_NoteOn,
				       channel, port, pitch, noteVelocity),
		     time,
		     TSE3::MidiCommand(TSE3::MidiCommand_NoteOff,
				       channel, port, pitch, noteVelocity),
		     time+duration));
  /*
    phraseEdit.insert
    (TSE3::MidiEvent(TSE3::MidiCommand(TSE3::MidiCommand_NoteOn,
    channel, port, pitch, noteVelocity),
    time));

    //  note += pitch;//delta[n];
    time += duration;
    phraseEdit.insert
    (TSE3::MidiEvent(TSE3::MidiCommand(TSE3::MidiCommand_NoteOff,
    channel, port, pitch, 0),
    time));*/

  noOfNotesToPlay++;
}


void TSEOSSMidiOutput::playChord()
{
  //  debuginfo("TSEOSS: play");
  if((noOfNotesToPlay>0)&&isOn())
    {
      int mypid; // FORK FORK FORK

      usleep(10); // if this function is called more times rapidly
      // (like in the midi-config test)
      // the old child will become a zombie (it doesn't return)
      if ((mypid=fork())==0)
	{  
	  debuginfo("Hi, I'm FORKED...");

	  phraseEdit.tidy(); // (should not be indispensible...)
	  TSE3::Song    song(1);
	  TSE3::Phrase *phrase = phraseEdit.createPhrase(song.phraseList());
	  TSE3::Part   *part   = new TSE3::Part(0, phraseEdit.lastClock());
	  part->setPhrase(phrase);
	  song[0]->insert(part);
	  part->params()->setVolume(noteVelocity*127/100);
	  // TSE3 Volume Parameter values (0-127, -1 = none, -2 = force none)


	  transport->play(&song, 0);
	  while (transport->status() != TSE3::Transport::Resting)
	    {
	      transport->poll();
	      usleep(10);
	      // perhaps sleep here to prevent slaughtering the CPU
	    }

	  debuginfo("... FORKED ending.");
	  _exit(0);  // EXIT CHILD PROCESS AFTER PLAYING A CHORD
	}
    }
  phraseEdit.reset();
  noOfNotesToPlay=0;
}



void TSEOSSMidiOutput::setupPatches()
{
  (*(*mixer)[midiCurrentDevice])[midiCurrentChannel]->setProgram(midiPatch);
  debuginfo("midi patch set");
}

bool TSEOSSMidiOutput::turnOn()
{
  internal_midifd=0;
  return true;
}


void TSEOSSMidiOutput::turnOff()
{
  internal_midifd=-1;
}



bool TSEOSSMidiOutput::mixerIsOpen()
{
  return true;
}

void TSEOSSMidiOutput::mixerSetMasterVolume(int newvol)
{
  midiCurrentMasterVolume=newvol;
  if(midiCurrentMasterVolume>100) midiCurrentMasterVolume=100;
  if(midiCurrentMasterVolume<0) midiCurrentMasterVolume=0;
}

void TSEOSSMidiOutput::mixerSetMidiVolume(int newvol)
{
  midiCurrentVolume=newvol;
  if (midiCurrentVolume>100) midiCurrentVolume=100;
  if (midiCurrentVolume<0) midiCurrentVolume=0;
  //(*mixer)[midiCurrentDevice]->setVolume(midiCurrentVolume, true);
  (*(*mixer)[midiCurrentDevice])[midiCurrentChannel]->setVolume(midiCurrentVolume);
  // what if I don't send the command??
  debuginfo("midi volume changed");
}

int  TSEOSSMidiOutput::mixerMasterVolume()
{
  return midiCurrentMasterVolume;
}

int  TSEOSSMidiOutput::mixerMidiVolume()
{
  return midiCurrentVolume;
}

void TSEOSSMidiOutput::setMidiCurrentDevice(int newval)
{
  if (unsigned(newval) > scheduler->numPorts())
    newval=scheduler->numPorts();
  if (scheduler->numPorts() > 0)
    port=scheduler->portNumber(newval);
  else
    port=TSE3::MidiCommand::NoPort;

  midiCurrentDevice=newval;

  debuginfo(QString("TSE MidiOutput: now using port %1").arg(port));
}


TSEAlsaMidiOutput::TSEAlsaMidiOutput()
  : TSEOSSMidiOutput(TSE3::Plt::UnixMidiSchedulerFactory::UnixPlatform_Alsa)
{
  midiOutputName=QString("ALSA/TSE");
}

TSEAlsaMidiOutput::~TSEAlsaMidiOutput(){}


TSEaRtsMidiOutput::TSEaRtsMidiOutput()
  : TSEOSSMidiOutput(TSE3::Plt::UnixMidiSchedulerFactory::UnixPlatform_Arts)
{
  midiOutputName=QString("aRts/TSE");
}

TSEaRtsMidiOutput::~TSEaRtsMidiOutput(){}

#endif //WITH_TSE3//HAVE_TSE3_TSE3_H
