/***************************************************************************
 ** $Id: midi-TSE.h,v 1.4 2004/05/13 02:57:20 wiecko Exp $
                          klearnnotes2
                          midi-OSS.h -  description
                             -------------------
    begin                : Sun Feb 29 23:58:22 CET 2004
    copyright            : (C) 2004 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <config.h>
#ifdef WITH_TSE3 //; HAVE_TSE3_TSE3_H

#ifndef KLN_MIDI_TSE3
#define KLN_MIDI_TSE3

#include "midi.h"

// for creating a new (one-chord) song:
#include <tse3/Song.h>
#include <tse3/PhraseEdit.h>
#include <tse3/Part.h>
#include <tse3/Track.h>
#include <tse3/MidiParams.h>

//for playing the song:
#include <tse3/Metronome.h>
//#include <tse3/util/MidiScheduler.h>
#include <tse3/Transport.h>
#include <tse3/plt/Factory.h>
#include <tse3/plt/OSS.h>
#include <tse3/Mixer.h>


// Specifies the root MIDI note of the scale
const int rootNote = 0x60;

// Duration of each note in the scale
const int duration = TSE3::Clock::PPQN;



class TSEOSSMidiOutput : public MidiOutput
{
 public:
  TSEOSSMidiOutput(TSE3::Plt::UnixMidiSchedulerFactory::UnixPlatform platform=
		   TSE3::Plt::UnixMidiSchedulerFactory::UnixPlatform_OSS);
  virtual ~TSEOSSMidiOutput();

  // overridden stuff:
  //! Opens /dev/sequencer and does basic setup. Returns true if successful.
  virtual bool turnOn();
  //! Closes /dev/sequencer
  virtual void turnOff();


  //! adds (schedules) one note to be played in next chord (in future)
  /*! pitch is a MIDI pitch (tune number) */
  virtual void addNoteToChord(int pitch);// for polyphony: you first add
  //                                        notes ...
  //! plays all scheduled notes as a chord
  virtual void playChord();//                ... and then play whole chord


  //! Returns true if /dev/mixer or other apropriate "device" can be opened
  virtual bool mixerIsOpen();
  //! Sets mixer master volume
  virtual void mixerSetMasterVolume(int newvol);
  //! Sets mixer midi volume
  virtual void mixerSetMidiVolume(int newvol);
  //! Returns current master volume.
  virtual int  mixerMasterVolume();
  //! Returns current midi volume.
  virtual int  mixerMidiVolume();

  virtual void setMidiCurrentDevice(int);

  //! Updated patches of all channels.
  /*! Should be called when midiPatch changes. Always run when MIDI
    sound is turned on.
  */
  virtual void setupPatches();

 protected:

  int noOfNotesToPlay;


  TSE3::PhraseEdit phraseEdit;
  int              note ;
  TSE3::Clock      time ;


  int              port;
  TSE3::Metronome            metronome;
  //    TSE3::Util::StreamMidiScheduler scheduler;
  TSE3::MidiScheduler       *scheduler;
  TSE3::Transport           *transport;
  TSE3::Mixer               *mixer;

};

///////////////////
class TSEAlsaMidiOutput : public TSEOSSMidiOutput
{
 public:
  TSEAlsaMidiOutput();
  virtual ~TSEAlsaMidiOutput();

  /*protected:
    TSE3::Plt::OSSMidiScheduler     scheduler;*/
};

///////////////////
class TSEaRtsMidiOutput : public TSEOSSMidiOutput
{
 public:
  TSEaRtsMidiOutput();
  virtual ~TSEaRtsMidiOutput();

  /*protected:
    TSE3::Plt::OSSMidiScheduler     scheduler;*/
};


#endif //KLN_MIDI_OSS_H

#endif  //WITH_TSE3//HAVE_TSE3_TSE3_H
