/***************************************************************************
 ** $Id: midi-timidity.cpp,v 1.2 2004/03/01 04:55:39 wiecko Exp $
                          klearnnotes2
                          midi-OSS.cpp -  description
                             -------------------
    begin                : Sun Feb 29 23:58:22 CET 2004
    copyright            : (C) 2004 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <qstring.h>
#include <unistd.h> // _exit()
//#include <stdlib.h>   //-
#include <fcntl.h>    //O_WRONLY and friends
#include <sys/ioctl.h>      // for writing to the device 
#include "klearnnotes2-defs.h"
#include "globals.h"
#include "midi-timidity.h" // this includes midi!


#include "timidity-io.c"

#define MIDI_DEV "/dev/sequencer"


// standard code from incude/linux/soundcard.h :
extern int midifd ; //= -1;
//SEQ_DEFINEBUF(1024);
//! standard OSS/Free code from incude/linux/soundcard.h
extern void seqbuf_dump();
//{
//  if (_seqbufptr)
//    write (midifd, _seqbuf, _seqbufptr);
//  _seqbufptr = 0;
//}


//! cretator; sets up default values
TimidityMidiOutput::TimidityMidiOutput() : MidiOutput()
{
  //  debuginfo("TimidityMidiOutput creator: start");
  internal_midifd = -1;
  mixerfd=-1;
  
  midiOutputName=QString("Timidity server");
  //  debuginfo("TimidityMidiOutput creator: end");

  timidityServerProcess=new QProcess();
  timidityServerProcess->clearArguments ();
  timidityServerProcess->addArgument( "timidity" );
  timidityServerProcess->addArgument( "-ir" );
  timidityServerProcess->addArgument( "7777" );
  debuginfo(QString("timidity server will be started by a command :")+
	    "\n timidity -ir 7777");
}

TimidityMidiOutput::~TimidityMidiOutput(){}

void TimidityMidiOutput::addNoteToChord(int pitch)
{
  if(::midifd==-1)//device is NOT opened
    return;
  notesToPlay[noOfNotesToPlay]=pitch;
  noOfNotesToPlay=(noOfNotesToPlay+1)%maxNotesInAChord;
}

void TimidityMidiOutput::playChord()
{
  if((noOfNotesToPlay>0)&&isOn())
    {
      int mypid; // FORK FORK FORK 
      if ((mypid=fork())==0)
	{  
	  //	  debugtxt("Playing FORKED...");
	  int newVel=60;
	  int midiCurrentChannel=0;
	  for(int i=0;i<noOfNotesToPlay;i++)
	    {
	      SEQ_START_NOTE(midiCurrentDevice, midiCurrentChannel,
			     notesToPlay[i], newVel);
	      midiCurrentChannel=(midiCurrentChannel+1)%midiMaxChan;
	      // notes are added to different channels
	    }
	  SEQ_DELTA_TIME(noteLength);
	  seqbuf_dump();        
	  timidity_ioctl(midifd, SNDCTL_SEQ_SYNC);
	  midiCurrentChannel=0;
	  for(int i=0;i<noOfNotesToPlay;i++)
	    {
	      SEQ_STOP_NOTE(midiCurrentDevice, midiCurrentChannel,
			    notesToPlay[i], 64);
	      // Uses 0x80, true MIDI note-off 
	      midiCurrentChannel=(midiCurrentChannel+1)%midiMaxChan;
	    }
	  seqbuf_dump();
	  timidity_ioctl(midifd, SNDCTL_SEQ_SYNC);
	  //	  debugtxt("... FORKED ending.");
	  _exit(0);  // EXIT CHILD PROCESS AFTER PLAYING A CHORD
	}
    }
  noOfNotesToPlay=0;
}



void TimidityMidiOutput::setupPatches()
{
  for(int i =0; i< midiMaxChan; i++)
    SEQ_SET_PATCH(midiCurrentDevice,i,midiPatch);
}

bool TimidityMidiOutput::turnOn()
{
  if (isOn()) 
    {
      //      debuginfo("midi_turnOn(): but midi IS already on!");
      return true;
    }
  suspendArts();
  //  debuginfo("MIDI: OPENING DEVICE!");

  startTimidityServer();

  if ((midifd = timidity_open(MIDI_DEV, O_WRONLY)) == -1)
    {
      //      debuginfo("midi_turnOn(): couldn't open midi device!");
      internal_midifd=-1;
      return false;     //  Can't open MIDI device
    }
  internal_midifd=midifd;
  //  debuginfo("MIDI: device is open! Starting timer.");
  SEQ_START_TIMER();
  //  debuginfo("MIDI: Setting tempo.");
  SEQ_SET_TEMPO(100);
  //  debuginfo("MIDI: Setting up patches.");
  setupPatches();
  //  debuginfo("MIDI: opening mixer.");
  if( (mixerfd = open("/dev/mixer", O_WRONLY, 0))==-1 )
    {
      //      debuginfo("midi_turnOn(): couldn't open mixer device!");
      return true; // can't open mixer but /dev/sequencer is open!
    }
  //  debuginfo("MIDI: setting up mixer.");
  mixerSetMasterVolume(midiCurrentMasterVolume);
  mixerSetMidiVolume(midiCurrentVolume);
  //  debuginfo("midi_turnOn(): midi and mixer open");
  return true;
}


void TimidityMidiOutput::turnOff()
{
  if(midifd!=-1) // otherwise timidity 'close' hangs :(
    timidity_close(midifd);
  midifd=-1;
  internal_midifd=-1;
  if (mixerfd != -1)
    {
      close(mixerfd);
      mixerfd=-1;
    }
  stopTimidityServer();
}



bool TimidityMidiOutput::mixerIsOpen()
{
  if(mixerfd==-1)
    {
      return false;
    }
  return true;
}

void TimidityMidiOutput::mixerSetMasterVolume(int newvol)
{
  midiCurrentMasterVolume=newvol;
  if(midiCurrentMasterVolume>100) midiCurrentMasterVolume=100;
  if(midiCurrentMasterVolume<0) midiCurrentMasterVolume=0;
  if(mixerfd!=-1)
    {
      int level = (midiCurrentMasterVolume << 8) + midiCurrentMasterVolume;
      ioctl( mixerfd, MIXER_WRITE(SOUND_MIXER_VOLUME),   &level );
      midiCurrentMasterVolume=level & 0xff;
      // above = returned value for left voulume;
      // right = (level & 0xff00) >> 8;
    }
}

void TimidityMidiOutput::mixerSetMidiVolume(int newvol)
{
  midiCurrentVolume=newvol;
  if (midiCurrentVolume>100) midiCurrentVolume=100;
  if (midiCurrentVolume<0) midiCurrentVolume=0;
  if(mixerfd!=-1)
    {
      int level = (midiCurrentVolume << 8) + midiCurrentVolume;
      ioctl( mixerfd, MIXER_WRITE(SOUND_MIXER_PCM),   &level );
      midiCurrentMasterVolume=level & 0xff;
      // above = returned value for left voulume;
      // right = (level & 0xff00) >> 8;
    }
}

int  TimidityMidiOutput::mixerMasterVolume()
{
  if(mixerIsOpen())
    {
      int level;
      ioctl( mixerfd, MIXER_READ(SOUND_MIXER_VOLUME), &level );
      if((level& 0xff) != midiCurrentMasterVolume)
	{
	  //	  printerror("Master volume changed by some other program!\n using new settings");
	  midiCurrentMasterVolume=level& 0xff;
	}
    }
  return midiCurrentMasterVolume;
}

int  TimidityMidiOutput::mixerMidiVolume()
{
  if(mixerIsOpen())
    {
      int level;
      ioctl( mixerfd, MIXER_READ(SOUND_MIXER_PCM), &level );
      if((level& 0xff) != midiCurrentMasterVolume)
	{
	  //	  printerror("Midi volume changed by some other program!\n using new settings");
	  midiCurrentMasterVolume=level& 0xff;
	}
    }
  return midiCurrentVolume;
}

void TimidityMidiOutput::startTimidityServer()
{
  //  debuginfo("starting timidity server");
  if(!  timidityServerProcess->isRunning())
    {
      //        debuginfo("really starting ...");
            if(timidityServerProcess->start())
	      {
		//debuginfo("... started OK");
	      }
    }
  KLN_usleep(1000000); //1s
  debuginfo("timidity server should be running now!");
  timidityServerProcess->closeStdin ();
}

void TimidityMidiOutput::stopTimidityServer()
{
  //  debuginfo("stopping timidity server");
  timidityServerProcess->tryTerminate();
  KLN_usleep(500000); //.5s
  if(timidityServerProcess->isRunning ())
    {
      debuginfo("KILLING timidity server...");
      timidityServerProcess->kill();
      KLN_usleep(500000); //.5s
    }
}
