/***************************************************************************
 ** $Id: midi-timidity.h,v 1.2 2004/05/13 02:57:20 wiecko Exp $
                          klearnnotes2
                          midi-OSS.h -  description
                             -------------------
    begin                : Sun Feb 29 23:58:22 CET 2004
    copyright            : (C) 2004 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KLN_MIDI_TIM_H
#define KLN_MIDI_TIM_H


#ifndef SEQUENCER_DEV
#define SEQUENCER_DEV "/dev/sequencer"
#endif

#include "midi.h"
#include <qprocess.h>

class TimidityMidiOutput : public MidiOutput
{
 public:
  TimidityMidiOutput();
  virtual ~TimidityMidiOutput();

  // overridden stuff:
  //! Opens /dev/sequencer and does basic setup. Returns true if successful.
  virtual bool turnOn();
  //! Closes /dev/sequencer
  virtual void turnOff();


  //! adds (schedules) one note to be played in next chord (in future)
  /*! pitch is a MIDI pitch (tune number) */
  virtual void addNoteToChord(int pitch);// for polyphony: you first add 
  //                                        notes ...
  //! plays all scheduled notes as a chord
  virtual void playChord();//                ... and then play whole chord


  //! Returns true if /dev/mixer or other apropriate "device" can be opened
  virtual bool mixerIsOpen();
  //! Sets mixer master volume
  virtual void mixerSetMasterVolume(int newvol);
  //! Sets mixer AUDIO (PCM!) volume
  /*! For timidity output audio volume has the same role, as midi volume for
    /dev/sequencer; therefore the name of the method has not been changed 
    (from kln2 point of view, this simply refers to MIDI sounds output volume,
    just because of usage of timidity the sound is send to audio device, not
    midi device). */
  virtual void mixerSetMidiVolume(int newvol);
  //! Returns current master volume.
  virtual int  mixerMasterVolume();
  //! Returns current AUDIO (PCM!) volume.
  /*! See notes at mixerSetMidiVolume(). */
  virtual int  mixerMidiVolume();

  //! Updated patches of all channels.
  /*! Should be called when midiPatch changes. Always run when MIDI
    sound is turned on.
  */
  virtual void setupPatches();

protected:

  void startTimidityServer();
  void stopTimidityServer();
  QProcess* timidityServerProcess;

  static const int maxNotesInAChord=40 ;
  //                                ^ this size should be >= no. of channels
  int notesToPlay[maxNotesInAChord]; // notes in one chord;
  int noOfNotesToPlay;
};
#endif //KLN_MIDI_TIM_H
