/***************************************************************************
 ** $Id: midi.h,v 1.5 2004/05/13 02:57:20 wiecko Exp $
                          klearnnotes2
                          midi.h -  description
                             -------------------
    begin                : Wed Oct  8 15:32:54 CEST 2003
    copyright            : (C) 2003 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KLN_MIDI_H
#define KLN_MIDI_H
// SOUND

#define defMIDIdevNum 0        // initial device number 
#define defMIDIchan 16         // initial no of MIDI channels 
#define defMASTERvol 100       // master volume
#define defMIDIvol 100         // midi volume



//! Inherited parent midi output class
/*! A class used to standarize access to different midi output ways/devices.
  It will be inherited to incorporate OSS/Free and timidity midi output.

  NOTE: to simplify following description I will refer to different  midi
  output ways as "way". I hope this will not introduce a confusion.

  Also, this holds mixer (volume) settings (because kind of mixer depends on
  kind of midiOutput way).

  In future ALSA, kmid and other support should be added in a similar way.
*/
class MidiOutput
{
 public:
  MidiOutput();
  virtual ~MidiOutput();
  //! name of the way
  virtual QString getMidiOutputName();
  //! fake initialization of midi output way
  /*! This always returns true.

  Should be overridden in such a way, that if successfull returns
  true, otherwise false */
  virtual bool turnOn();
  //! fake close/turn off the way
  virtual void turnOff();
  //! true if the way has been initialized and ready to use
  /*! This simply checks if internal_midifd != -1 .*/
  virtual bool isOn();


  //! fake adds (schedules) one note to be played in next chord (in future)
  /*! This does nothing.

  Should be overridden in such a way, that one note of pitch pitch
  should to be played in next chord. In such a case pitch would be
  a MIDI pitch (tune number) */
  virtual void addNoteToChord(int pitch);// for polyphony: you first add 
  //                                        notes ...
  //! fake plays all scheduled notes as a chord
  /*! This does nothing. Should be overridden.*/
  virtual void playChord();//                ... and then play whole chord


  //! sets note's/chord's tune length
  virtual void setNoteLength(int newLength);
  //! returns current note's/chord's tune length
  virtual int  getNoteLength();
  //! sets note's/chord's tune volume (loudness)
  virtual void setNoteVelocity(int newVelocity);
  //! returns current note's/chord's tune volume
  virtual int  getNoteVelocity();
  //! sets MIDI patch
  virtual void setMidiPatch(int newPatch);
  //! returns MIDI patch
  virtual int  getMidiPatch();
  
  //! Returns true if /dev/mixer or other apropriate "device" can be opened
  virtual bool mixerIsOpen();
  //! Sets mixer master volume
  virtual void mixerSetMasterVolume(int newvol);
  //! Sets mixer midi volume
  virtual void mixerSetMidiVolume(int newvol);
  //! Returns current master volume.
  virtual int  mixerMasterVolume();
  //! Returns current midi volume.
  virtual int  mixerMidiVolume();

  virtual int getMidiMaxChan();// max no of channels
  virtual int getMidiCurrentChannel();
  virtual int getMidiCurrentDevice();
  virtual void setMidiMaxChan(int);// max no of channels
  virtual void setMidiCurrentChannel(int);
  virtual void setMidiCurrentDevice(int);
  virtual int getMidiCurrentVolume();
  //  virtual void setMidiCurrentVolume(int);
  virtual int getMidiCurrentMasterVolume();
  //  virtual void setMidiCurrentMasterVolume(int);

  //! checks if we should try to suspend aRts and if so suspends it
  /*! Otherwise aRts (KDE sound server) blocks music /dev/ices */
  void suspendArts();

 protected:

  //! Updated patches of all channels.
  /*! Should be called when midiPatch changes. Always run when MIDI
    sound is turned on.
  */
  virtual void setupPatches();

  //! name of the way
  QString midiOutputName;
  //! true if the way has been initialized and ready to use
  //  bool isOn;
  //! note's/chord's tune length
  static int noteLength;
  //! note's/chord's tune volume (loudness)
  static int noteVelocity;
  //! midi patch
  static int midiPatch;
  //! -1 means midi is has not been opened
  int internal_midifd;
  //! -1 means mixer is has not been opened
  int mixerfd;

  int midiMaxChan;// max no of channels
  int midiCurrentChannel;
  int midiCurrentDevice;
  int midiCurrentVolume;
  int midiCurrentMasterVolume;

  //! should we try to suspend Arts?
  static bool tryToSuspendArts;
};

#endif //KLN_MIDI_H
