/***************************************************************************
 ** $Id: midi_setup.cpp,v 1.11 2004/05/13 02:57:20 wiecko Exp $
                          klearnnotes2
                          midi_setup.cpp -  description
                             -------------------
    begin                : Wed Oct  8 15:32:54 CEST 2003
    copyright            : (C) 2003 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <config.h>
#include "klearnnotes2.h"
#include "midi_setup.h"
#include "midi-OSS.h"
#ifdef WITH_TSE3 //HAVE_TSE3_TSE3_H
#include "midi-TSE.h"
#endif // WITH_TSE3 //HAVE_TSE3_TSE3_H
#include "midi-timidity.h"

#include <qvariant.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qtranslator.h>
#include <qcombobox.h>

MIDI_setup::MIDI_setup(QPixmap pix, QWidget* parent,  const char* name, 
		       bool modal, WFlags fl )
  : QDialog( parent, name, modal, fl )
  // note: pix is a pixmap for What'sThis button
{
  if ( !name )
    setName( "MIDI_setup" );
  //  resize( 580, 226 ); 
  setCaption( tr( "MIDI sound setup" ) );
  midi_setupLayout = new QGridLayout( this, 1, 1, 11, 6, "midi_setupLayout"); 

  textLabelDeviceType = new QLabel(this);
  textLabelDeviceType->setText(tr("MIDI system &output:"));
  midi_setupLayout->addWidget( textLabelDeviceType, 0, 0 );
  comboOutputWay =  new QComboBox(this);
  comboOutputWay -> insertItem("OSS/Free");
  comboOutputWay -> insertItem("timidity server");
#ifdef WITH_TSE3//HAVE_TSE3_TSE3_H
  comboOutputWay -> insertItem("OSS/TSE");
  comboOutputWay -> insertItem("ALSA/TSE");
  comboOutputWay -> insertItem("aRts/TSE");
#endif
  if (midiOutput->getMidiOutputName()==QString("OSS/Free"))
    comboOutputWay->setCurrentItem(0);
  else if(midiOutput->getMidiOutputName()==QString("Timidity server"))
    comboOutputWay->setCurrentItem(1);
#ifdef WITH_TSE3//HAVE_TSE3_TSE3_H
  else if(midiOutput->getMidiOutputName()==QString("OSS/TSE"))
    comboOutputWay->setCurrentItem(2);
  else if(midiOutput->getMidiOutputName()==QString("ALSA/TSE"))
    comboOutputWay->setCurrentItem(3);
  else
    comboOutputWay->setCurrentItem(4);
#endif
  midi_setupLayout->addWidget(comboOutputWay , 0, 1 );
  textLabelDeviceType->setBuddy(comboOutputWay);

  textLabelMidiDevNum = new QLabel(this);
  textLabelMidiDevNum->setText( tr( "MIDI device &number" ) );
  midi_setupLayout->addWidget( textLabelMidiDevNum, 1, 0 );
  spinBoxDeviceNumber = new QSpinBox( this, "spinBoxDeviceNumber" );
  spinBoxDeviceNumber->setMaxValue( 5 );
  spinBoxDeviceNumber->setMinValue( 0 );
  spinBoxDeviceNumber->setValue(midiOutput->getMidiCurrentDevice()  );
  QWhatsThis::add( spinBoxDeviceNumber, 
		   tr( "If you do have a midi sequencer but you "
		       "don't hear sound - experiment with this"
		       " setting. Check if you hear sound after "
		       "pressing \"Test\"." ) );
  midi_setupLayout->addWidget( spinBoxDeviceNumber, 1, 1 );
  textLabelMidiDevNum->setBuddy(spinBoxDeviceNumber);
  textLabelEmpty = new QLabel( this);
  textLabelEmpty->setText( "" );
  midi_setupLayout->addMultiCellWidget( textLabelEmpty, 1, 1, 2, 4 );
  pushButtonTest = new QPushButton( this, "pushButtonTest" );
  pushButtonTest->setText( tr( "Test" ) );
  QWhatsThis::add( pushButtonTest, tr("You should hear 2 chords!"));
  midi_setupLayout->addWidget( pushButtonTest, 1, 5 );


  textLabelNumChan = new QLabel( this );
  textLabelNumChan->setText( tr( "Number of &channels" ) );
  midi_setupLayout->addWidget( textLabelNumChan, 2, 0 );
  spinBoxNumberOfChannels = new QSpinBox( this, "SpinBoxNumberOfChannels" );
  spinBoxNumberOfChannels->setMinValue( 16 );
  spinBoxNumberOfChannels->setMaxValue( 128 );
  spinBoxNumberOfChannels->setValue( midiOutput->getMidiMaxChan() );
  QWhatsThis::add( spinBoxNumberOfChannels, 
		   tr("Number of channels for midi output.\n"
		      "\n"
		      "The standard value is 16."));
  midi_setupLayout->addWidget( spinBoxNumberOfChannels, 2, 1 );
  textLabelNumChan->setBuddy(spinBoxNumberOfChannels);


  textLabelMasteVolume = new QLabel( this );
  textLabelMasteVolume->setText( tr( "Master &Volume" ) );
  midi_setupLayout->addWidget( textLabelMasteVolume, 4, 0 );
  spinBoxVolumeMaster = new QSpinBox( this, "spinBoxVolumeMaster" );
  spinBoxVolumeMaster->setMaxValue( 100 );
  spinBoxVolumeMaster->setValue( midiOutput->mixerMasterVolume() );
  spinBoxVolumeMaster->setEnabled( midiOutput->mixerIsOpen() );
  midi_setupLayout->addWidget( spinBoxVolumeMaster, 4, 1 );
  textLabelMasteVolume->setBuddy(spinBoxVolumeMaster);
  textLabelMasterVolPerc = new QLabel( this );
  textLabelMasterVolPerc->setText( tr( "%" ) );
  midi_setupLayout->addWidget( textLabelMasterVolPerc, 4, 2 );
  QWhatsThis::add( spinBoxVolumeMaster, 
		   tr("This is volume affecting all sound output, "
		      "e.g. music form CD, mp3 players, MIDI. "
		      "To get maximum volume both: Master Volume "
		      "and Midi Volume have to be at 100%."));


  textLabelMidiVol = new QLabel( this, "textLabelMidiVol" );
  textLabelMidiVol->setText( tr( "MIDI Vo&lume" ) );
  midi_setupLayout->addWidget( textLabelMidiVol, 5, 0 );
  spinBoxVolumeMidi = new QSpinBox( this, "spinBoxVolumeMidi" );
  spinBoxVolumeMidi->setMaxValue( 100 );
  spinBoxVolumeMidi->setValue( midiOutput->getMidiCurrentVolume() );
  midi_setupLayout->addWidget( spinBoxVolumeMidi, 5, 1 );
  textLabelMidiVol->setBuddy(spinBoxVolumeMidi);
  textLabelMidiVolPerc = new QLabel( this, "TextLabelMidiVolPerc" );
  textLabelMidiVolPerc->setText( tr( "%" ) );
  midi_setupLayout->addWidget( textLabelMidiVolPerc, 5, 2 );
  QWhatsThis::add( spinBoxVolumeMidi, 
		   tr("This is volume affecting only MIDI output;"
		      "all sound output (e.g. music form CD, "
		      "mp3 players) is not affected. "
		      "To get maximum volume both: Master Volume "
		      "and Midi Volume have to be at 100%. "
		      "Note, that for timidity this changes mixer "
		      "AUDIO volume, and NOT MIDI volume!"));



  textLabelNoteLength = new QLabel( this );
  textLabelNoteLength->setText( tr( "Note &length" ) );
  midi_setupLayout->addWidget( textLabelNoteLength, 6, 0 );
  spinBoxNoteLength = new QSpinBox( this, "spinBoxNoteLength" );
  spinBoxNoteLength->setMaxValue( 150 );
  spinBoxNoteLength->setMinValue( 20 );
  spinBoxNoteLength->setValue( midiOutput->getNoteLength() );
  midi_setupLayout->addWidget( spinBoxNoteLength, 6, 1 );
  textLabelNoteLength->setBuddy(spinBoxNoteLength);
  textLabelMilliseconds = new QLabel( this );
  textLabelMilliseconds->setText( tr( "ms" ) );
  midi_setupLayout->addWidget( textLabelMilliseconds, 6, 2 );
  QWhatsThis::add( spinBoxNoteLength, 
		   tr(" length of a note played; \n"
		      " test it with the TEST button"));

  textLabelMidiPatch = new QLabel( this );
  textLabelMidiPatch->setText( tr( "MIDI patc&h" ) );
  midi_setupLayout->addWidget( textLabelMidiPatch, 7, 0 );
  spinBoxMidiPatch = new QSpinBox( this, "spinBoxMidiPatch" );
  spinBoxMidiPatch->setMaxValue( 127 );
  spinBoxMidiPatch->setValue( midiOutput->getMidiPatch() );
  midi_setupLayout->addWidget( spinBoxMidiPatch, 7, 1 );
  textLabelMidiPatch->setBuddy(spinBoxMidiPatch);
  QWhatsThis::add( spinBoxMidiPatch, tr("(0-127); 0=piano, 24=guitar" ) );


  textLabelOctaveBias = new QLabel( this);
  textLabelOctaveBias->setText( tr( "Octave &bias" ) );
  midi_setupLayout->addWidget( textLabelOctaveBias, 8, 0 );
  spinBoxOctaveBias = new QSpinBox( this, "spinBoxOctaveBias" );
  spinBoxOctaveBias->setMaxValue( 0 );
  spinBoxOctaveBias->setMinValue( -1 );
  spinBoxOctaveBias->setValue(OCTAVEbias  );
  QWhatsThis::add( spinBoxOctaveBias, 
		   tr("Standard notation for the guitar uses treble "
		      "clef, but each note means an one octave lower note."
		      "<UL>"
		      "<LI>If you set it to 0 you will hear standard sounds."
		      "<LI>If you set it to -1 you will hear sounds which "
		      "your guitar should make (i.e. one octave lower)"
		      "</UL>"
		      "<EM>Note: </EM> If you want to learn notes for the bass "
		      "clef, setting this to 0 is strongly recommended! "
		      "(with -1 sounds in the bass clef are REALLY low).")) ;
  midi_setupLayout->addWidget( spinBoxOctaveBias, 8, 1 );
  textLabelOctaveBias->setBuddy(spinBoxOctaveBias);
  pushButtonWhat = new QPushButton( this, "pushButtonWhat" );
  pushButtonWhat->setText( " " );
  pushButtonWhat->setPixmap(pix);
  midi_setupLayout->addWidget( pushButtonWhat, 8, 5 ,Qt::AlignLeft);


  pushButtonGuitar = new QPushButton( this, "pushButtonGuitar" );
  pushButtonGuitar->setText( tr( "Set &Guitar defaults" ) );
  midi_setupLayout->addWidget( pushButtonGuitar, 9, 0 );
  pushButtonNonGuitar = new QPushButton( this, "pushButtonNonGuitar" );
  pushButtonNonGuitar->setText( tr( "Set non-guitar &defaults" ) );
  midi_setupLayout->addMultiCellWidget( pushButtonNonGuitar, 9, 9, 1, 2 );
  pushButtonOK = new QPushButton( this, "pushButtonOK" );
  pushButtonOK->setText( tr( "OK" ) );
  midi_setupLayout->addWidget( pushButtonOK, 9, 3 );
  pushButtonCancel = new QPushButton( this, "pushButtonCancel" );
  pushButtonCancel->setText( tr( "Cancel" ) );
  midi_setupLayout->addWidget( pushButtonCancel, 9, 4 );

  // signals and slots connections
  connect(pushButtonGuitar,SIGNAL(clicked()),
	  this,SLOT(pushButton_guitar_clicked()));
  connect(pushButtonNonGuitar,SIGNAL(clicked()),
	  this,SLOT(pushButton_nonguitar_clicked()));
  connect(pushButtonOK,SIGNAL(clicked()),
	  this,SLOT(pushButton_OK_clicked()));
  connect(pushButtonCancel,SIGNAL(clicked()),
	  this,SLOT(pushButton_cancel_clicked()));
  connect(comboOutputWay,SIGNAL(activated(int)),
	  this,SLOT(comboOutputWay_activated(int)));
  connect(spinBoxDeviceNumber,SIGNAL(valueChanged(int)),
	  this,SLOT(spinBox_device_changed(int)));
  connect(spinBoxNumberOfChannels,SIGNAL(valueChanged(int)),
	  this,SLOT(spinBox_channels_changed(int)));
  connect(spinBoxVolumeMaster,SIGNAL(valueChanged(int)),
	  this,SLOT(spinBox_master_volume_changed(int)));
  connect(spinBoxVolumeMidi,SIGNAL(valueChanged(int)),
	  this,SLOT(spinBox_midi_volume_changed(int)));
  connect(spinBoxNoteLength,SIGNAL(valueChanged(int)),
	  this,SLOT(spinBox_notelength_changed(int)));
  connect(spinBoxMidiPatch,SIGNAL(valueChanged(int)),
	  this,SLOT(spinBox_midi_patch_changed(int)));
  connect(spinBoxOctaveBias,SIGNAL(valueChanged(int)),
	  this,SLOT(spinBox_octave_changed(int)));
  connect(pushButtonTest,SIGNAL(clicked()),
	  this,SLOT(pushButton_test_clicked()));
  connect(pushButtonWhat,SIGNAL(clicked()),
	  this,SLOT(pushButton_what_clicked()));
  connect(spinBoxNumberOfChannels,SIGNAL(valueChanged(int)),
	  this,SLOT(spinBox_octave_changed(int)));

  // remember old settings:
  olddevice=midiOutput->getMidiCurrentDevice();
  oldchan=midiOutput->getMidiMaxChan();
  oldMasterVolume=midiOutput->mixerMasterVolume();
  oldMidiVolume=midiOutput->mixerMidiVolume();
  oldlength=midiOutput->getNoteLength();
  oldpatch=midiOutput->getMidiPatch();
  oldoctave=OCTAVEbias;
}

MIDI_setup::~MIDI_setup() { }

void MIDI_setup::pushButton_guitar_clicked()
{
  spinBoxMidiPatch->setValue(24);
  spinBox_midi_patch_changed(24);
  spinBoxOctaveBias->setValue(-1);
  spinBox_octave_changed(-1);
}

void MIDI_setup::pushButton_nonguitar_clicked()
{
  spinBoxMidiPatch->setValue(0);
  spinBox_midi_patch_changed(0);
  spinBoxOctaveBias->setValue(0);
  spinBox_octave_changed(0);
}

void MIDI_setup::pushButton_OK_clicked()
{
  accept();
}

void MIDI_setup::pushButton_cancel_clicked()
{
  //restore settings
  midiOutput->setMidiCurrentDevice( olddevice );
  midiOutput->setMidiMaxChan( oldchan);
  midiOutput->mixerSetMasterVolume(oldMasterVolume);
  midiOutput->mixerSetMidiVolume(oldMidiVolume);
  midiOutput->setNoteLength( oldlength);
  midiOutput->setMidiPatch( oldpatch);
  OCTAVEbias= oldoctave;
  reject();
}

void MIDI_setup::spinBox_device_changed( int newdevice)
{
  midiOutput->setMidiCurrentDevice(newdevice);
}

void MIDI_setup::spinBox_channels_changed(int newchan)
{
  midiOutput->setMidiMaxChan(newchan);
}

void MIDI_setup::spinBox_master_volume_changed( int newVol)
{
  midiOutput->mixerSetMasterVolume(newVol);
}

void MIDI_setup::spinBox_midi_volume_changed( int newVol)
{
  midiOutput->mixerSetMidiVolume(newVol);
}


void MIDI_setup::spinBox_notelength_changed( int newLen)
{
  midiOutput->setNoteLength(newLen);
}

void MIDI_setup::spinBox_midi_patch_changed( int newPatch)
{
  midiOutput->setMidiPatch(newPatch);
  //  midiOutput->setupPatches();
}

void MIDI_setup::spinBox_octave_changed( int newBias)
{
  OCTAVEbias=newBias;
}

void MIDI_setup::pushButton_test_clicked()
{
  //play some music: 2 chords:
  //  CEG
  //   EGC1
  int c=60;
  int d=c+2;
  int e=d+2;
  int f=e+1;
  int g=f+2;
  int a=g+2;
  int h=a+2;
  int c1=h+1;
  midiOutput->addNoteToChord(g);
  midiOutput->addNoteToChord(e);
  midiOutput->addNoteToChord(c);
  midiOutput->playChord();
  midiOutput->addNoteToChord(c1);
  midiOutput->addNoteToChord(g);
  midiOutput->addNoteToChord(e);
  midiOutput->playChord();
}

void MIDI_setup::pushButton_what_clicked()
{
  QWhatsThis::enterWhatsThisMode();
}

void MIDI_setup::comboOutputWay_activated(int activatedItem)
{
  if((activatedItem==0)&&
     (midiOutput->getMidiOutputName()!=QString("OSS/Free")))
    {
      debuginfo("ending timidity, starting OSS");
      midiOutput->turnOff();
      delete midiOutput;
      midiOutput=new OSSMidiOutput();
      midiOutput->turnOn();
      return;
    }
  if((activatedItem==1)&&
     (midiOutput->getMidiOutputName()!=QString("timidity server")))
    {
      debuginfo("ending OSS, starting timidity");
      midiOutput->turnOff();
      delete midiOutput;
      midiOutput=new TimidityMidiOutput();
      midiOutput->turnOn();
      return;
    }
#ifdef WITH_TSE3
  if((activatedItem==2)&&
     (midiOutput->getMidiOutputName()!=QString("OSS/TSE")))
    {
      debuginfo("ending whatever, starting OSS TSE");
      midiOutput->turnOff();
      delete midiOutput;
      midiOutput=new TSEOSSMidiOutput();
      midiOutput->turnOn();
      return;
    }
  if((activatedItem==3)&&
     (midiOutput->getMidiOutputName()!=QString("ALSA/TSE")))
    {
      debuginfo("ending whatever, starting ALSA TSE");
      midiOutput->turnOff();
      delete midiOutput;
      midiOutput=new TSEAlsaMidiOutput();
      midiOutput->turnOn();
      return;
    }
  if((activatedItem==4)&&
     (midiOutput->getMidiOutputName()!=QString("aRts/TSE")))
    {
      debuginfo("ending whatever, starting aRts TSE");
      midiOutput->turnOff();
      delete midiOutput;
      midiOutput=new TSEaRtsMidiOutput();
      midiOutput->turnOn();
      return;
    }
#endif

  debuginfo("OSS vs timidity vs TSE: nothing changed!");
}
