/***************************************************************************
 ** $Id: midi_setup.h,v 1.5 2004/03/01 04:27:54 wiecko Exp $
                          klearnnotes2
                          midi_setup.h -  description
                             -------------------
    begin                : Wed Oct  8 15:32:54 CEST 2003
    copyright            : (C) 2003 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MIDI_SETUP_H
#define MIDI_SETUP_H

#include <qvariant.h>
#include <qdialog.h>
#include <qpixmap.h>
class QVBoxLayout; 
class QHBoxLayout; 
class QGridLayout; 
class QLabel;
class QPushButton;
class QSpinBox;
class QComboBox;

//! The sound options setup dialog.
class MIDI_setup : public QDialog
{ 
  Q_OBJECT

    public:
//! constructor
/*! pmapForWhatsThisButton is a workaround: predefined Qt whatsThisButton is
	available only in the main window toolbar. We "borrow" here its pixmap
	to create a custom whatsThisButton for sound setup dialog.
*/
  MIDI_setup(QPixmap pmapForWhatsThisButton, QWidget* parent = 0,
	     const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
  ~MIDI_setup();

  //! midi output way: OSS or timidity
  QComboBox* comboOutputWay;

//! midi device number
/*! FIXIT: user should see device's name too (easy to do). */
  QSpinBox* spinBoxDeviceNumber; 

//! number of channels 
/*! FIXIT: whatis comments are wrong (there are always at least 16 channels).*/
  QSpinBox* spinBoxNumberOfChannels;

//! master Volume
  QSpinBox* spinBoxVolumeMaster; 
//! midi Volume
  QSpinBox* spinBoxVolumeMidi;

//! the length of note's tune played
  QSpinBox* spinBoxNoteLength;

//! midi patch
  QSpinBox* spinBoxMidiPatch; 
//! octave bias
/*! If set to -1 makes note's tunes one octave lower (guitar tuning). */
  QSpinBox* spinBoxOctaveBias; 

  public slots:
//! copies guitar defaults (midi patch=24 and octave bias =-1)
    virtual void pushButton_guitar_clicked();
//! copies non-guitar defaults (midi patch=0 and octave bias =0)
  virtual void pushButton_nonguitar_clicked();

  virtual void pushButton_OK_clicked();
  virtual void pushButton_cancel_clicked();

  virtual void comboOutputWay_activated(int activatedItem);
  virtual void spinBox_device_changed( int newDevice );
  virtual void spinBox_channels_changed(int newChan);
  virtual void spinBox_master_volume_changed( int );
  virtual void spinBox_midi_volume_changed( int );
  virtual void spinBox_notelength_changed( int newLength);
  virtual void spinBox_midi_patch_changed( int newPatch);
  virtual void spinBox_octave_changed( int newBias);
  virtual void pushButton_test_clicked();
  virtual void pushButton_what_clicked();

 protected:
  QGridLayout* midi_setupLayout;

 private:
  QLabel* textLabelEmpty;

  QLabel* textLabelDeviceType;
  QLabel* textLabelMidiDevNum;
  QLabel* textLabelNumChan;

  QLabel* textLabelMasteVolume;
  QLabel* textLabelMasterVolPerc;

  QLabel* textLabelMidiVol;
  QLabel* textLabelMidiVolPerc;

  QLabel* textLabelNoteLength;
  QLabel* textLabelMilliseconds;
  QLabel* textLabelMidiPatch;
  QLabel* textLabelOctaveBias;

//! Test button: plays a chord to test sound settings.
  QPushButton* pushButtonTest;
//! "What's this?" button
  QPushButton* pushButtonWhat;
//! "guitar defaults" button
  QPushButton* pushButtonGuitar;
//! "non-guitar defaults" button
  QPushButton* pushButtonNonGuitar;
//! OK
  QPushButton* pushButtonOK;
//! Cancel
  QPushButton* pushButtonCancel;

//! device number value saved; restored if an user cancels
  int olddevice;
//! number of channels value saved; restored if an user cancels
  int oldchan;
//! MIDI voluume value saved; restored if an user cancels
  int oldMidiVolume;
//! master volume value saved; restored if an user cancels
  int oldMasterVolume;
//! note's tune length value saved; restored if an user cancels
  int oldlength;
//! MIDI patch value saved; restored if an user cancels
  int oldpatch;
//! octave bias value saved; restored if an user cancels
  int oldoctave;
};

#endif // MIDI_SETUP_H
