/***************************************************************************
 ** $Id: musicwidget.cpp,v 1.5 2004/05/13 02:57:20 wiecko Exp $
                          klearnnotes2
                          musicwidget.cpp -  description
                             -------------------
    begin                : Tue Mar  2 02:41:00 CET 2004
    copyright            : (C) 2004 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../config.h"

#include "musicwidget.h"
#include "KLNcanvasitems.h"
#include "klearnnotes2.h"
#include <qpointarray.h> 
#include <qtimer.h>
#include "globals.h"
#include <qapplication.h>
#include <qwhatsthis.h>
#include <qtooltip.h>

PianoKeyboard::PianoKeyboard(Exercise* exercise,QWidget * parent,
			     int leftmostPitch, int howManyKeys,
			     int howWide,
			     KLearnNotes2 * mainwidget)
  : RandomObjectsContainer(parent)//parent)
{
  //  debuginfo("PianoKeyboard() - start");

  canvas=new QCanvas(howWide,10);
  canvasView=new ClickableView(canvas, this,  mainwidget);
 
  firstPitch=leftmostPitch;
  numberOfKeys=howManyKeys;
  graphScale=KLNmath_floor(0.5*double(howWide)/
			   double(positionOfAPitch(numberOfKeys+firstPitch)-
				  positionOfAPitch(firstPitch)));
  //  DEBUG_VAR(graphScale);
  //  DEBUG_VAR(positionOfAPitch(numberOfKeys+firstPitch)-positionOfAPitch(firstPitch)+4);

  // because of flooring above, the actuall width may be smaller:
  int trueWidth=graphScale*(positionOfAPitch(numberOfKeys+firstPitch)-
			    positionOfAPitch(firstPitch))*2+5;

  canvas->resize(trueWidth,2*11*graphScale);
  canvas->setBackgroundColor(QColor(147,77,2));
  canvasView->setVScrollBarMode(QScrollView::AlwaysOff);
  canvasView->setHScrollBarMode(QScrollView::AlwaysOff);
  canvasView->setFixedWidth(trueWidth);
  setFixedWidth(trueWidth+2*5);// QFrame
  canvasView->setFixedHeight(15*graphScale);
  setFixedHeight(15*graphScale+2*25); //QFrame

  for(int i=0;i<numberOfKeys;i++)
    {
      keys.append(new PianoKeyboardKey(exercise,canvas,firstPitch+i,
				       positionOfAPitch(firstPitch+i)-
				       positionOfAPitch(firstPitch),
				       graphScale));
      keys.last()->show();
      allQuestions.append(keys.last());
      connect(keys.at(i), SIGNAL(signalNoteActivated(BaseMusicNote*)),
	      this, SLOT(slotNoteActivated(BaseMusicNote*)));
      connect(keys.at(i), SIGNAL(signalPitchActivated(int)),
	      this, SLOT(slotPitchActivated(int)));
    }

  rangeSlider1 = new QSlider(positionOfAPitch(leftmostPitch),
			     positionOfAPitch(leftmostPitch+howManyKeys-1), 1, 
			     positionOfAPitch(leftmostPitch),
			     Qt::Horizontal,this );
  oldSlider1Position=positionOfAPitch(leftmostPitch);
  rangeSlider1->setTickmarks(QSlider::Both);
  rangeSlider1->setFixedWidth( trueWidth-10);
  rangeSlider2 = new QSlider(positionOfAPitch(leftmostPitch),
			     positionOfAPitch(leftmostPitch+howManyKeys-1), 1, 
			     positionOfAPitch(leftmostPitch+howManyKeys-1),
			     Qt::Horizontal,this);
  oldSlider2Position=positionOfAPitch(leftmostPitch+howManyKeys-1);
  rangeSlider2->setTickmarks(QSlider::Both);
  rangeSlider2->setFixedWidth( trueWidth-10);
  QWhatsThis::add(rangeSlider1 , 
		  tr("<B><U>Range sliders</U></B> :<BR>"
		     "choose range of <EM>testnotes</EM> <BR>"));
  QToolTip::add( rangeSlider1,tr("Range slider 1"));
  QWhatsThis::add(rangeSlider2,
		  tr("<B><U>Range sliders</U></B> :<BR>"
		     "choose range of <EM>testnotes</EM> <BR>"));
  QToolTip::add( rangeSlider2,tr("Range slider 2"));


  myLayout = new QVBoxLayout(this);  
  hLayout1 = new QHBoxLayout(myLayout);  
  hLayout1->addStretch(10);
  hLayout1->addWidget(rangeSlider1);
  hLayout1->addSpacing(5);
  hLayout1->addStretch(10);
  hLayout2 = new QHBoxLayout(myLayout);  
  hLayout2->addStretch(10);
  hLayout2->addWidget(rangeSlider2);
  hLayout2->addSpacing(5);
  hLayout2->addStretch(10);
  hLayout3 = new QHBoxLayout(myLayout);  
  hLayout3->addStretch(10);
  hLayout3->addWidget(canvasView);
  hLayout3->addStretch(10);
  //  debuginfo("PianoKeyboard() - end");
  connect(rangeSlider1,SIGNAL(valueChanged(int)),
	  this, SLOT(slider1Slided(int)));
  connect(rangeSlider2,SIGNAL(valueChanged(int)),
	  this, SLOT(slider2Slided(int)));
}


void PianoKeyboard::slotNoteActivated(BaseMusicNote* activatedNote)
{
  debuginfo(QString("Emiting answerByNote(%1)"
		    ).arg(activatedNote->stringName()));
  emit answerByNote(activatedNote);
}

void PianoKeyboard::slotPitchActivated(int activatedPitch)
{
  debuginfo(QString("Emiting answerByPitch(%1)").arg(activatedPitch));
  BaseMusicNote* note=new BaseMusicNote(activatedPitch);
  //  emit answerByPitch(activatedPitch);
  emit answerByPitch(note);
}

  
void PianoKeyboard::highlightPitch(BaseMusicNote* noteGivingPitch,
				   char color , 
				   int time )
{
  int num=noteGivingPitch->pitch();
  num-=firstPitch;
  if((num<0)||(num>numberOfKeys-1))
    return;

  (keys.at(num))->highlight(color,time);
}

//! highlights all keys of a given name
void PianoKeyboard::highlightAllOfName(BaseMusicNote* noteGivingName,
				       char color , 
				       int time )
{
  //  debuginfo("highlighting piano keyboard: start");
  int num=noteGivingName->pitch();
  num-=firstPitch;
  //  DEBUG_VAR(num);
  int numtmp=num+12;

  while(num>=0){
    if((num<numberOfKeys-1)){
      (keys.at(num))->highlight(color,time); //GRR should be somehow hltname
    }
    num-=12;
    //    DEBUG_VAR(num);
  }

  num=numtmp;
  //  DEBUG_VAR(num);
  while(num<numberOfKeys-1){
    if(num>=0){
      (keys.at(num))->highlight(color,time); //GRR should be somehow hltname
    }
    num+=12;
    //    DEBUG_VAR(num);
  }
}



void PianoKeyboard::setScale(double newScale){newScale++;}
void PianoKeyboard::setWidth(double newWidth){newWidth++;}
void PianoKeyboard::enablePitch(BaseMusicNote* noteGivingPitch)
{
  noteGivingPitch++;
}
void PianoKeyboard::disablePitch(BaseMusicNote* noteGivingPitch)
{
  noteGivingPitch++;
}

void PianoKeyboard::enableSharps(){}
void PianoKeyboard::enableFlats(){}
void PianoKeyboard::disableSharps(){}
void PianoKeyboard::disableFlats(){}

bool PianoKeyboard::isActivePitch(BaseMusicNote* note)
{
  //  debuginfo(QString("Checking pitch: %1").arg(note->pitch()));
  int pos=positionOfAPitch(note->pitch());
  if(((pos-rangeSlider1->value())*(pos-rangeSlider2->value()))>0)
    {
      //      debuginfo("false");
      return false;
    }
  //  debuginfo("true");
  return true;
}

int  PianoKeyboard::positionOfAPitch(int pitch)
{
  int numInOct=BaseMusicNote::numOfNoteInOctave(pitch);
  if(numInOct>4)
    numInOct++;
  return (int(pitch/12)*14 +  numInOct);
}

int  PianoKeyboard::pitchOfAPosition(int position)
{
  int num=position%14;
  if((num==5)||(num==13)){return -1;}
  if(num>5) {num --;}
  return  int(position/14)*12+num;
}

void PianoKeyboard::slider1Slided(int newValue)
{
  //  DEBUG_VAR(newValue);
  //  DEBUG_VAR(oldSlider1Position);
  int num=newValue%14;
  //  DEBUG_VAR(num);
  //  debuginfo("");
  if((num==5)||(num==13))
    {
      int d=((newValue-oldSlider1Position>0)?1:-1); //step one more step
      //      DEBUG_VAR(d);
      oldSlider1Position=newValue+d;
      rangeSlider1->setValue(newValue+d);
      return;
    }
  oldSlider1Position=newValue;
  emit activeNotesChanged();
}

void PianoKeyboard::slider2Slided(int newValue)
{
  //  DEBUG_VAR(newValue);
  //  DEBUG_VAR(oldSlider2Position);
  int num=newValue%14;
  //  DEBUG_VAR(num);
  //  debuginfo("");
  if((num==5)||(num==13))
    {
      int d=((newValue-oldSlider2Position>0)?1:-1); //step one more step
      //      DEBUG_VAR(d);
      rangeSlider2->setValue(newValue+d);
      oldSlider2Position=newValue+d;
      return;
    }
  oldSlider2Position=newValue;
  emit activeNotesChanged();
}

//void PianoKeyboard::setupActiveNotes()
//{
// * go through all keys
// * force each to check if it is active
// * if 'yes' add to active notes
//}

bool PianoKeyboard::checkAnswerByName(BaseMusicNote* note)
{
  bool result;
  if(((PianoKeyboardKey*)currentQuestion)->checkIfNameMatches(note))
    {
      result=true;
      ((PianoKeyboardKey*)currentQuestion)->highlight('G',0);
      currentQuestion->addCorrect(timeSpent);
      debuginfo("emitting: Correct answer");
      emit comment("correct answer");
      emit answerWasRight(((PianoKeyboardKey*)currentQuestion)->getOneNote(),timeSpent,true);
    }
  else
    {
      result=false;
      ((PianoKeyboardKey*)currentQuestion)->highlight('R',0);
      currentQuestion->addWrong(timeSpent);
      schedule();
      debuginfo("emitting: Wrong answer");
      emit comment("wrong answer");
      emit answerWasRight(((PianoKeyboardKey*)currentQuestion)->getOneNote(),timeSpent,false);
    }
  KLN_usleep(500000);//0.5 s 
  currentQuestion->hideForTest(); // resets red/green highlight
  questionNumber++;
  DEBUG_VAR(questionNumber);
  if(questionNumber <testLength)
    {
      nextQuestion();
      currentQuestion->showForTest();
      timeSpent=0;
      qApp->processEvents () ;
    }
  else
    {
      stopTest();
      // THIS HAS TO RUN EXERCISE's STOPTEST!!!!!
    }
  return result;// NOT REALLY NEEDED AND RETURNED ONLY AFTER NEW QUESTION
  // IS CHOSEN AND SHOWN!
}

void PianoKeyboard::updateNotesNames()
{
  for(PianoKeyboardKey* keyIterator= keys.first();
      keyIterator;
      keyIterator=keys.next())
    keyIterator->updateNotesNames();
}

int PianoKeyboardKey::rtti() const
{
  return rttiNum;
}

PianoKeyboardKey::PianoKeyboardKey(Exercise* exercise,
				   QCanvas* argCanvas, int pitch, 
				   int position, int scale)
  : QObject(argCanvas),QCanvasPolygon(argCanvas), RandomObject(exercise)
{
  graphScale=scale;
  canvas=argCanvas;
  int x=int(position*2*scale);
  QPointArray* points=new QPointArray();
  if(! BaseMusicNote::isWhite(pitch)) // BLACK KEY
    {
      //      debuginfo("Black");
      x+=scale;
      points->setPoints(4,
			x+1,1,
			x+1,8*scale-1,
			x+2*scale-1,8*scale-1,
			x+2*scale-1,1);
      //    1  .___.
      //       |   |
      //       |   |
      //       |   |
      // 8s-1  |___|
      //     x+1  x+2s-1
      //
      //      setPen(QPen(QColor(0,0,50),1));
      labelx=x+scale;
      labely=8*scale-3;
      setBrush(QBrush(QColor(0,0,0)));      
      notes.append(new BaseMusicNote(pitch-1,1));
      notes.append(new BaseMusicNote(pitch+1,-1));
    }
  else     // WHITE KEY
    {
      //      debuginfo("White:");
      //      setPen(QPen(QColor(50,0,0),1));
      setBrush(QBrush(QColor(245,245,245)));
      switch(BaseMusicNote::pitchToWhiteNumberInOctave(pitch))
	{
	case 0:
	case 3:
	  // b KEY   ( |_ )
	  //	  debuginfo("White: |_");
	  points->setPoints(6,
			    x+1,1,
			    x+1,14*scale-1,
			    x+4*scale-1,14*scale-1,
			    x+4*scale-1,8*scale+1,
			    x+3*scale-1,8*scale+1,
			    x+3*scale-1,1);
	  //    1  .___.
	  //       |   |   x+4s-1
	  // 8s-1  |   |___. 
	  //       |       |
	  // 14s-1 |___^___| 
	  //     x+1  x+2s-1
	  labelx=x+2*scale;
	  notes.append(new BaseMusicNote(pitch,0));
	  notes.append(new BaseMusicNote(pitch-1,+1));
	  //	  setBrush(QBrush(QColor(245,245,0)));
	  break;
	case 2:
	case 6:
	  // d KEY   ( _| )
	  //	  debuginfo("White: _|");
	  points->setPoints(6,
			    x+4*scale-1,1,
			    x+4*scale-1,14*scale-1,
			    x+1,14*scale-1,
			    x+1,8*scale+1,
			    x+scale+1,8*scale+1,
			    x+scale+1,1);

	  //              x+4s-1
	  //    1      .___.
	  //           |   | 
	  // 8s-1  .___|   | 
	  //       |       |
	  // 14s-1 |___^___| 
	  //     x+1  x+2s-1
	  labelx=x+2*scale;
	  notes.append(new BaseMusicNote(pitch,0));
	  notes.append(new BaseMusicNote(pitch+1,-1));
	  //	  setBrush(QBrush(QColor(245,0,245)));
	  break;
	default:
	  // A KEY   (  _|_ )
	  //	  debuginfo("White: _|_");
	  points->setPoints(8,
			    x+3*scale-1,1,
			    x+3*scale-1,8*scale+1,
			    x+4*scale-1,8*scale+1,
			    x+4*scale-1,14*scale-1,
			    x+1,14*scale-1,
			    x+1,8*scale+1,
			    x+scale+1,8*scale+1,
			    x+scale+1,1);
	  //             x+3s-1
	  //    1     .__.
	  //          |  | 
	  // 8s+1  .__|  |__. 
	  //       |        |
	  // 14s-1 |__^_____| 
	  //     x+1 x+s-1
	  labelx=x+2*scale;
	  notes.append(new BaseMusicNote(pitch,0));
	}
      labely=14*scale-3; // for white keys
    }
  setPoints(*points);
  sharpsActive=true;
  flatsActive=true;
  setZ(pitch);
  // if autodeletion is on removing pointer from a list deletes the object too
  nameLabels.setAutoDelete(true);
  nameLabelsBackground.setAutoDelete(true);
  for(BaseMusicNote* noteIterator = notes.first();
      noteIterator; 
      noteIterator = notes.next() )
    activeNotes.append(noteIterator);

  //  checkIfIsActive();//this sets up activeNotes!!! IT HAS TO BE RUN BY
  // EXERCISE BEFORE USING THIS WIDGET
}

/// NOOOOOOOOOOOTE: THERE MAY BE A PROBLEM: THERE IS A isActive()
// METHOD OF (I think) QFrame ! Possible bug: check if it is not used
// insead of isActiveForTest!

void PianoKeyboardKey::clicked()
{
  if(RandomObject::active)
    {
      BaseMusicNote* noteIterator;
      for ( noteIterator = activeNotes.first(); noteIterator; 
	    noteIterator = activeNotes.next() )
	{
	  if(noteIterator->getAccidental()==0)
	    emit signalNoteActivated(noteIterator);
	  if((noteIterator->getAccidental()==1)&&sharpsActive)
	    emit signalNoteActivated(noteIterator);
	  if((noteIterator->getAccidental()==-1)&&flatsActive)
	    emit signalNoteActivated(noteIterator);
	}
      noteIterator = activeNotes.first();
      if(noteIterator)
	emit signalPitchActivated(noteIterator->pitch());
    }
}

void PianoKeyboardKey::highlight(char color,int time)
{
  QColor qc,qcDark;
  switch(color)
    {
    case 'R':
      qc=QColor(255,200,200);
      qcDark=QColor(255,0,0);
      break;
    case 'G':
      qc=QColor(196,255,200);
      qcDark=QColor(0,100,0);
      break;
    default:
      //	case 'B':
      qc=QColor(196,239,255);
      qcDark=QColor(0,0,255);
    }

  if(isActiveForTest())
    {
      //      debuginfo("highlighting specific key");
      setBrush(QBrush(qc));

      for(  QCanvasText* txtIterator = nameLabels.first();
	    txtIterator; 
	    txtIterator = nameLabels.next() )
	{
	  txtIterator->setColor(qcDark);
	  txtIterator->show();
	}
      //       for(  QCanvasRectangle* rectIterator = nameLabelsBackground.first();
      // 	    rectIterator; 
      // 	    rectIterator = nameLabelsBackground.next() )
      // 	rectIterator->show();

    }
  canvas->update();
  if(time>0)
    QTimer::singleShot( /*TIMEOFHIGHLIGHT*/ time, this,SLOT(resetColor()));
}


void PianoKeyboardKey::resetColor()
{
  if((notes.first())->isWhite())
    if(isActiveForTest())
      setBrush(QBrush(QColor(245,245,245)));
    else
      setBrush(QBrush(QColor(165,165,165)));
  else
    setBrush(QBrush(QColor(0,0,0)));
  for(  QCanvasText* txtIterator = nameLabels.first();
	txtIterator; 
	txtIterator = nameLabels.next() )
    {
      txtIterator->hide();
    }
  for(  QCanvasRectangle* rectIterator = nameLabelsBackground.first();
	rectIterator; 
	rectIterator = nameLabelsBackground.next() )
    rectIterator->hide();
  canvas->update();
}

//! for future: will highlight and show ONE note name
/*! When there will be note's names displayed when the key is highlighted, 
  this will choose only one, apropriate name (contrary to highlight(), which
  will show them all. 

  For now simply runs highlight(). */
void PianoKeyboardKey::highlight(BaseMusicNote* note,char color, int time)
{
  highlight(color,time);
  //kill the warnings:
  note++;
}

bool PianoKeyboardKey::checkIfIsActive()
{
  //  debuginfo("Checking if it is active");
  activeNotes.clear();
  BaseMusicNote* noteIterator;
  bool check=false;
  for ( noteIterator = notes.first(); noteIterator; 
	noteIterator = notes.next() )
    {
      //      debuginfo("PianoKeyboardKey::checkIfIsActive()");
      //      DEBUG_VAR(noteIterator->pitch());
      //      DEBUG_VAR(noteIterator->stringName());
      if((myExercise->isActiveNote(noteIterator)))
	{
	  check=true;
	  activeNotes.append(noteIterator);
	}
    }
  //  noteIterator = notes.first();
  //  if(!(myExercise->isActivePitch(noteIterator)))
  //    check=false;

  //  if(check)
  //    {
  //      //we are active:
  //    }
  //  else
  //    {
  //      // we are not active: let's get grey
  //    }
  RandomObject::active=check;
  //  debuginfo(QString("Pitch %1 : key is %2 ."
  //		    ).arg(notes.first()->pitch()).arg((check)?"ON":"OFF"));
  resetColor();
  resetNameLabel();
  return(check);
}

void PianoKeyboardKey::resetNameLabel()
{
  // delete old labels and backgrounds
  nameLabels.clear(); // autodeletion is on - this deletes the items too!
  nameLabelsBackground.clear();

  if(!RandomObject::active)
    return; //the key is not active



  int num=1; //labels for different keys's names should be higher and higher
  for(  BaseMusicNote* noteIterator = activeNotes.first();
	noteIterator; 
	noteIterator = activeNotes.next() )
    {
      //      if((myExercise->isActiveName(noteIterator)))
      //	{
      QCanvasText* name=new QCanvasText(noteIterator->stringName(true),
					canvas );
      nameLabels.append(name);
      name->setFont( QFont( "Arial", int(2.5*graphScale), QFont::Bold ) );
      name->setColor( QColor( 0, 0, 0 ) );
      name->hide();
      name->setZ(2000);
	  
      QCanvasRectangle* back=new QCanvasRectangle(canvas);
      nameLabelsBackground.append(back);
      back->setSize(name->boundingRect().width()+2,
		    name->boundingRect().height()-2);
      //	  back->setBrush(QBrush(QColor(150,255,255)));
      //	  back->setPen(QPen (QColor (150,255,255),0));
      back->setBrush(QBrush(QColor(255,255,255)));
      back->setPen(QPen (QColor (255,255,255),0));
      back->setZ(name->z()-1);
      back->hide();
      // positions:
      back->move(labelx-back->boundingRect().width()/2,
		 labely-num*back->boundingRect().height()+1);
      name->move(labelx-name->boundingRect().width()/2-1,
		 labely-num*name->boundingRect().height()-2);
      num++;
      //	}
    }
}


void PianoKeyboardKey::showForTest()
{
  if(isActiveForTest())
    {
      setBrush(QBrush(QColor(196,239,255)));
    }
  canvas->update();
}
void PianoKeyboardKey::hideForTest()
{
  resetColor();
}

bool PianoKeyboardKey::checkIfNameMatches(BaseMusicNote* note)
{
  BaseMusicNote* noteIterator;
  for ( noteIterator = notes.first(); noteIterator; 
	noteIterator = notes.next() )
    {
      if(noteIterator->stringName()==note->stringName())
	return true;
    }
  return false;
}

BaseMusicNote* PianoKeyboardKey::getOneNote()
{
  return notes.first();
}

void PianoKeyboardKey::updateNotesNames()
{
  resetNameLabel();
}

