/***************************************************************************
 ** $Id: musicwidget.h,v 1.5 2004/05/13 02:57:20 wiecko Exp $
                          klearnnotes2
                          musicwidget.cpp -  description
                             -------------------
    begin                : Tue Mar  2 02:41:00 CET 2004
    copyright            : (C) 2004 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

 
#ifndef KLN_MUSICWIDGET_H
#define KLN_MUSICWIDGET_H

#include "../config.h"

#include "klearnnotes2-defs.h"

#include <qframe.h>
#include <qptrlist.h>
#include <qcanvas.h>
#include <qlayout.h>
#include <qslider.h>


#include "basemusicnote.h"
#include "randomobject.h"

class ClickableView;
class KLearnNotes2;
class Exercise;

//! one key of a piano keyboard
class PianoKeyboardKey : public QObject,  public QCanvasPolygon,
			 public RandomObject
{
  Q_OBJECT
    public:
  PianoKeyboardKey(Exercise*, QCanvas*, int pitch,int position, int scale);

  //! returns a canvas object recognition number, unique for PianoKeyboardKey
  virtual int rtti() const;

  //!  the PianoKeyboardKey canvas recognition number
  /*! if(object->rtti()==PianoKeyboardKey::rttiNum) whe know we deal with
    a  PianoKeyboardKey and not with, say QCanvasEllipse etc. */
  static const int rttiNum=98054;

  virtual bool checkIfIsActive();

  //! returns one of notes corresponing to the key
  /*! usefull only for getting key's pitch */
  virtual BaseMusicNote* getOneNote();

  public slots:
    //! note, that if time=0 it highlights permanently
    virtual void highlight(char color,int time);
  //!  highlight() with a choice: which name label to display
  virtual void highlight(BaseMusicNote* note,char color, int time);
  virtual void resetColor();
  virtual void clicked();
  //! re-implemented from RandomObject
  virtual void showForTest();
  //! re-implemented from RandomObject
  virtual void hideForTest();
  virtual bool checkIfNameMatches(BaseMusicNote*);
  //! connected to notationChanged signal
  virtual void updateNotesNames();

 signals:
  //! user clicked - used for both for off-test highlights and test answers
  void signalNoteActivated(BaseMusicNote*);
  //! user clicked - used for both for off-test highlights and test answers
  void signalPitchActivated(int);
       
 protected:
  //! all notes corresponding to this key
  QPtrList<BaseMusicNote> notes;
  //! active notes corresponding to this key 
  /*! this depends on other widgets' settings */
  QPtrList<BaseMusicNote> activeNotes;
  //! not used yet
  bool sharpsActive;
  //! not used yet
  bool flatsActive;
  //! canvas in which whole piano keyboard is displayed
  QCanvas* canvas;

  //! labels with names of activeNotes
  QPtrList<QCanvasText> nameLabels;
  //! backgrounds labels with names of activeNotes
  /*! not needed for white keys; maybe may be needed for black keys*/
  QPtrList<QCanvasRectangle> nameLabelsBackground;
  //! goes through active notes and sets nameLabels accordingly
  virtual void resetNameLabel();
  //! horizontal position of naming labels
  int labelx;
  //! vertical postion of lower line of the key
  int labely; 
  //! scale for graph objects - remembered to set right txt heights
  int graphScale;
};

class PianoKeyboard : public RandomObjectsContainer
{
  Q_OBJECT
    public:
  PianoKeyboard(Exercise* exercise,
		QWidget * parent,int leftmostPitch, int howManyKeys,
		int howWide, KLearnNotes2 * mainwidget);

 signals:
  //  void pitchActivated(int); USE void answerByPitch(BaseMusicNote*);
  // void noteActivated(BaseMusicNote*); void answerByNote(BaseMusicNote*);

  //! this informs parent that my sliders' position changed
  /*! and it should ask all widgets to setupActiveNotes */
  void activeNotesChanged();
  // this informs keys that either my or other sliders' position changed
  /* i.e. asks them to setupActiveNotes */
  //  void shouldSetupActiveNotes();

  public slots:
    //! highlights notes of the same pitch 
    /*! highlight=="change color for fixed time "

    notes=="all widgets corresponding to the notes"
    */
    virtual void highlightPitch(BaseMusicNote*,char color ='0', 
				int time = TIMEOFHIGHLIGHT);

  //! highlights notes of the same name as noteGivingName
  /*! the highlighted keys will be in one-octave separation */
  virtual void highlightAllOfName(BaseMusicNote* noteGivingName,
				  char color ='0', 
				  int time = TIMEOFHIGHLIGHT);

  //! sets scale for all graphical objects [not used yet]
  /*! this:
   * - hides QFrame
   * - changes scale of all childredn
   * - adjusts size
   * - shows the QFrame again
   */
  virtual void setScale(double newScale);
  //! [not used yet, I think]
  virtual void setWidth(double newWidth);

  //! enable* enable widgets corresponging to pitch
  virtual void enablePitch(BaseMusicNote*);
  //! disable* disable widgets corresponging to pitch
  virtual void disablePitch(BaseMusicNote*);

  //! [not used yet]
  virtual void enableSharps();
  //! [not used yet]
  virtual void enableFlats();
  //! [not used yet]
  virtual void disableSharps();
  //! [not used yet]
  virtual void disableFlats();

  //  virtual void initTest();
  //  virtual void nextQuestion();
  //  virtual void schedule();

  //  virtual void setupActiveNotes();

  virtual bool checkAnswerByName(BaseMusicNote*);

  virtual void updateNotesNames();

  protected slots:
    void slotNoteActivated(BaseMusicNote*);
  void slotPitchActivated(int pitch);
  void slider1Slided(int newValue);
  void slider2Slided(int newValue);
 

 public:
  //! should the pitch be considered active (==a testnote)?
  /*! MusicWidgets have features which choose active notes. For example
    button box has checkboxes for different note names, staff has range
    sliders.

    With this method a note can ask: "hi, I'm C1. Should I be considered 
    active?".

    The note is active only if it is active according to all Exercise's 
    widgets.
  */
  virtual bool isActivePitch(BaseMusicNote*);

 protected:
  //! all the keys of this keyboard
  QPtrList<PianoKeyboardKey> keys;
  //! the midi pitch of the leftmost key (60=middle C)
  int firstPitch;
  //! number of all keys in this keyboard
  int numberOfKeys;
  //! scale of graphical objects
  int graphScale;
  QCanvas* canvas;
  ClickableView* canvasView;  

  //! returns position of a key corresponding to a pitch
  /*! 1. number of full octaves from C0 is calculated and multiplied by 14
    2. numberOfNoteInOctave is calculated, and if it is greater than 4 it is 
    ++ed
    3. sum of the above is returned
  */
  int positionOfAPitch(int pitch);
  //! returns pitch corresponding to a position or -1 if invalid
  /*! reverse of positionOfAPitch */
  int pitchOfAPosition(int position);


  //! HBoxLayout
  QBoxLayout* myLayout;
  //! for slider1
  QBoxLayout* hLayout1;
  //! for slider2
  QBoxLayout* hLayout2;
  //! for the keyboard canvas view
  QBoxLayout* hLayout3;
  QSlider* rangeSlider1;
  QSlider* rangeSlider2;
  //! used to know direction of "skip"
  int oldSlider1Position;
  int oldSlider2Position;
};
#endif       // KLN_MUSICWIDGET_H
