/***************************************************************************
 ** $Id: oneletternotation.cpp,v 1.1 2004/04/11 06:25:55 fcuenca Exp $
                          klearnnotes2
                          oneletternotation.cpp
                             -------------------
    begin                : Sun Mar 7 22:13:54 CEST 2003
    copyright            : (C) 2003 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <assert.h>
#include <stdlib.h>

#include "oneletternotation.h"

#define NOTE_COUNT 7
#define ACCIDENT_COUNT 3

OneLetterNotation::OneLetterNotation(const char* noteLetters, const char* accidentLetters):
  _noteLetters(noteLetters), _accidentLetters(accidentLetters)
{
    assert(_noteLetters.length() == NOTE_COUNT);

    if(accidentLetters == NULL)
    {
        _accidentLetters = " #b";
    }
    
    assert(_accidentLetters.length() == ACCIDENT_COUNT);
}

QString OneLetterNotation::getNoteLabel(ScaleNote noteName, NoteAccident noteAcc) const
{
    assert(noteName >= 0 && noteName < NOTE_COUNT);
    assert(noteAcc >= 0 && noteAcc < ACCIDENT_COUNT);
    
    if(_accidentLetters[noteAcc] == ' ')
    {
        // for natural notes, return just the note name
        return QString(QChar(_noteLetters[noteName]));
    }
    else
    {
       return QString(QChar(_noteLetters[noteName])) + QString(QChar(_accidentLetters[noteAcc]));
    }
}

int OneLetterNotation::getNoteChar(ScaleNote noteName, NoteAccident noteAcc) const
{
    assert(noteName >= 0 && noteName < NOTE_COUNT);
    assert(noteAcc >= 0 && noteAcc < ACCIDENT_COUNT);
    
    return _noteLetters[noteName];
}

//-----------------------------------------------------------------------------

EnglishNotation::EnglishNotation(): OneLetterNotation("BAGFEDC")
{
}

//-----------------------------------------------------------------------------

GermanNotation::GermanNotation(): OneLetterNotation("HAGFEDC")
{
}


