/***************************************************************************
 ** $Id: oneletternotation.h,v 1.1 2004/04/11 06:25:55 fcuenca Exp $
                          klearnnotes2
                          oneletternotation.h
                             -------------------
    begin                : Sun Mar  7 22:13:54 CEST 2004
    copyright            : (C) 2003 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __ONELETTERNOTATION_H__
#define __ONELETTERNOTATION_H__

#include <qstring.h>
#include "scalenotation.h"

//! This class represent a notation where a single letter is used to name musical notes
/*! This class is intended to provide a general implementation for those notations, based on
    a list of letters provided as a simple char array.
*/
class OneLetterNotation: public ScaleNotation
{
public:
   //! Default constructs the notation
   /*! The noteLetters array must have 7 characters, with the notes in the same order as the enum
       values in the ScaleNotation::ScaleNote enumeration.
       The accidentLetters array must have 3 characters with the simbols in the same order as the
       enum values in the ScaleNote::NoteAccident enumeration.
  */
   OneLetterNotation(const char* noteLetters, const char* accidentLetters = NULL);
    
   virtual QString getNoteLabel(ScaleNote noteName, NoteAccident noteAcc = evNatural) const;
   virtual int     getNoteChar(ScaleNote noteName, NoteAccident noteAcc = evNatural) const;

private:
   QString _noteLetters;
   QString _accidentLetters;

};  

//! Class that implements the English notation
class EnglishNotation: public OneLetterNotation
{
public:
   EnglishNotation();        
};

//! Class that implements the German notation
class GermanNotation: public OneLetterNotation
{
public:
   GermanNotation();
};

#endif

