/***************************************************************************
 ** $Id: scalenotation.h,v 1.1 2004/04/11 06:25:55 fcuenca Exp $
                          klearnnotes2
                          scalenotation.h
                             -------------------
    begin                : Sun Mar  7 22:13:54 CEST 2004
    copyright            : (C) 2003 by Marek Wieckowski
    email                : wiecko AT users.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __SCALENOTATION_H__
#define __SCALENOTATION_H__

#include <qobject.h>
#include <qstring.h>

//! This class represents a specific notation for the notes in the musical scale
/*! Note that this is an abstract class that must be subclassed to create a concrete
    notation. 
*/
class ScaleNotation: public QObject
{
  Q_OBJECT

public:
   enum ScaleNote { evB = 0, evA, evG, evF, evE, evD, evC };
   enum NoteAccident { evNatural, evSharp, evFlat };

   //! Notation deinitializer
   /*! This class is intented as base for a hierarchy,  therefore requires a virtual destructor.
       The default implementation does nothing.
   */
   virtual ~ScaleNotation();

   //! Returns a short label for the given note
   /*! The short label usually is a letter or word and a simbol representing the accident.
       For example: C, Do, F#, Fa#
   */
   virtual QString getNoteLabel(ScaleNote noteName, NoteAccident noteAcc = evNatural) const = 0;

   //! Returns a more verbose label for the given note
   /*! The default implementation for this method uses the value returned by getNoteLabel
       (using Natural as accident) and then appends the words "sharp" or "flat" if necessary.
   */
   virtual QString getNoteLongLabel(ScaleNote noteName, NoteAccident noteAcc = evNatural) const;

   //! Returns a one character letter to represent the given note
   virtual int     getNoteChar(ScaleNote noteName, NoteAccident noteAcc = evNatural) const = 0;

};  

#endif

